#!/usr/bin/env python3
# type: ignore [misc]

"""
radiocc.
"""


import os
from pathlib import Path
from typing import Any

# import envtoml
import click
import ruamel.yaml
import yaml
from dotmap import DotMap
from pudb import set_trace as bp  # noqa: F401

from radiocc import NAME, VERSION, core, util
from radiocc.model import (
    Config,
    ConfigBuilder,
    ConfigKeysOpt,
    ConfigKeysReq,
    Folders,
)
from radiocc.util import (  # print_error,
    form_yes_or_no,
    format_error,
    print_info,
    print_validation,
    raise_error,
)

# MANIFEST_FILE = Path("pyproject.toml")
# MANIFEST = DotMap(envtoml.load(MANIFEST_FILE.open()))
# NAME = MANIFEST.tool.poetry.name
# VERSION = MANIFEST.tool.poetry.version

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])

FILE_PATH = Path(os.path.realpath(__file__))
SOURCES_PATH = FILE_PATH.parent
ASSETS_PATH = SOURCES_PATH / "assets"
CONFIG_PATH = ASSETS_PATH / "config.yaml"
DEFAULT_INFORMATION_PATH = ASSETS_PATH / Folders.information.name
DEFAULT_TO_PROCESS_PATH = Path(Folders.to_process.name)
DEFAULT_RESULTS_PATH = Path(Folders.results.name)
DEFAULT_CLI_STR = "\r"
DEFAULT_CLI_STR_FLAG = "\r\r"
REPLACEMENT_DEFAULT_CLI_STR_FLAG = None


def version(ctx: Any, param: Any, value: Any) -> None:
    if not value or ctx.resilient_parsing:
        return
    click.echo(VERSION)
    ctx.exit()


@click.group(context_settings=CONTEXT_SETTINGS, invoke_without_command=True)
@click.pass_context
@click.option(
    "-v",
    "--version",
    is_flag=True,
    callback=version,
    expose_value=False,
    is_eager=True,
    help="Show version.",
)
def cli(ctx: click.Context) -> None:
    """Radio occultations."""
    # Share default command options.
    ctx.ensure_object(dict)
    # ctx.obj["var"] = var

    if ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())
        ctx.exit()


@cli.command()
@click.option(
    "-g", "--gui", is_flag=True, help="Start the program with the graphical interface."
)
@click.pass_context
def run(
    ctx: click.Context,
    gui: bool,
) -> None:
    """Run the program."""
    run_gui() if gui else run_cli()


def run_cli() -> None:
    """Start the CLI program."""
    CFGF = read_config_file()
    CFG = parse_config_file(CFGF)
    check_config_folders_correct(CFG)

    core.initialization(CFG)


def run_gui() -> None:
    """Start the graphical interface."""
    CFGF = read_config_file()
    CFG = parse_config_file(CFGF)
    check_config_folders_correct(CFG)

    raise_error("the graphical interface is not implemented yet.")


@cli.command(name="config")
@click.pass_context
@click.option("--file", is_flag=True, help="Show the content of the config file.")
@click.option(
    "--information",
    is_flag=False,
    flag_value=DEFAULT_CLI_STR_FLAG,
    default=DEFAULT_CLI_STR,
    help="Set the path to the folder `information`.",
)
@click.option(
    "--to-process",
    "to_process",
    is_flag=False,
    flag_value=DEFAULT_CLI_STR_FLAG,
    default=DEFAULT_CLI_STR,
    help="Set the path to the folder `to_process`.",
)
@click.option(
    "--results",
    is_flag=False,
    flag_value=DEFAULT_CLI_STR_FLAG,
    default=DEFAULT_CLI_STR,
    help="Set the path to the folder `results`.",
)
@click.option(
    "--folders",
    is_flag=False,
    flag_value=DEFAULT_CLI_STR_FLAG,
    default=DEFAULT_CLI_STR,
    help="Set the path to the folders.",
)
def config_(
    ctx: click.Context,
    file: bool,
    information: str,
    to_process: str,
    results: str,
    folders: str,
) -> None:
    """Interact with the configurations."""
    # This function runs if no argument is not provided.
    config_noarg(ctx, file, information, to_process, results, folders)

    if file:
        config_file()

    config_edit(information, to_process, results, folders)


def config_noarg(
    ctx: click.Context,
    file: str,
    information: str,
    to_process: str,
    results: str,
    folders: str,
) -> None:
    """Show the config formatted if no argument is provided."""
    if (
        not file
        and information == DEFAULT_CLI_STR
        and to_process == DEFAULT_CLI_STR
        and results == DEFAULT_CLI_STR
        and folders == DEFAULT_CLI_STR
    ):
        config()
        exit()


def config() -> None:
    """Show the config formatted."""
    CFGF = read_config_file()
    CFG = parse_config_file(CFGF)

    print("Configurations")
    print(str(CFG))

    check_config_folders_correct(CFG)


def config_file() -> None:
    """Show the config file."""
    CFGF = read_config_file()

    # Change all DEFAULT_CLI_STR_FLAG to None and all None to empty
    for (KEY, VALUE) in CFGF.items():
        if VALUE == DEFAULT_CLI_STR_FLAG or VALUE is None:
            CFGF[KEY] = ""

    print(f"Config file @{CONFIG_PATH}")
    print("\n".join(f"{attribute}: {value}" for (attribute, value) in CFGF.items()))

    CFG = parse_config_file(CFGF)
    check_config_folders_correct(CFG)
    exit()


def config_edit(
    information: str = DEFAULT_CLI_STR,
    to_process: str = DEFAULT_CLI_STR,
    results: str = DEFAULT_CLI_STR,
    folders: str = DEFAULT_CLI_STR,
) -> None:
    """Edit the config file."""
    update = False
    CFGF = read_config_file()

    # Check if `information` variable has been asked to be changed.
    for (KEY, VALUE) in dict(
        information=information, to_process=to_process, results=results, folders=folders
    ).items():
        if VALUE != DEFAULT_CLI_STR:
            CFGF[KEY] = VALUE
            print_validation(
                f"CONFIG: the variable `{KEY}` has been changed to {VALUE}"
            )
            update = True

    # Change all DEFAULT_CLI_STR_FLAG to None and all None to empty
    for (KEY, VALUE) in CFGF.items():
        if VALUE == DEFAULT_CLI_STR_FLAG:
            CFGF[KEY] = None
    util.yaml_add_representer_none()

    # Update config file.
    if update:
        with open(CONFIG_PATH, "w") as fp:
            yaml.dump(CFGF.toDict(), fp)

    # Read config file again to check if it is correct.
    CFGF = read_config_file()
    CFG = parse_config_file(CFGF)
    check_config_folders_correct(CFG)


def config_information(information: str) -> None:
    """Edit the config information variable."""
    config_edit(information=information)


def config_to_process(to_process: str) -> None:
    """Edit the config to_process variable."""
    config_edit(to_process=to_process)


def config_results(results: str) -> None:
    """Edit the config results variable."""
    config_edit(results=results)


def config_folders(folders: str) -> None:
    """Edit the config folders variable."""
    config_edit(folders=folders)


def config_get() -> Config:
    """Get the config."""
    CFGF = read_config_file()
    CFG = parse_config_file(CFGF)
    check_config_folders_correct(CFG)
    return CFG


def read_config_file() -> DotMap:
    """Load the config file."""
    CFGF_DICT, IND, BSI = ruamel.yaml.util.load_yaml_guess_indent(open(CONFIG_PATH))
    return DotMap(CFGF_DICT)


def parse_config_file(CFGF: DotMap) -> Config:
    """Check that the config file is correct."""
    # List the unused required variables.
    MISSING_VARIABLES = [KEY.name for KEY in list(ConfigKeysReq) if KEY not in CFGF]

    # Raise an error if any required variable is missing.
    if MISSING_VARIABLES:
        # as_list is true if at least two variables are missing, for plural syntax
        as_list = len(MISSING_VARIABLES) > 1
        raise_error(
            "the config file is missing the%s variable%s"
            % (("se", "s: ") if as_list else ("", ""))
            + ("\n- " if as_list else ": ").join([""] + MISSING_VARIABLES)
            + ("" if as_list else ".")
        )

    # Create the config object from its builder.
    cfg = ConfigBuilder()

    # Check if `folders` has been set.
    if ConfigKeysOpt.folders.name in CFGF:
        cfg.information = Path(CFGF.folders)
        cfg.to_process = Path(CFGF.folders)
        cfg.results = Path(CFGF.folders)

    # Check if information, to_process and results paths have been set.
    for KEY in list(Folders):
        if KEY.name in CFGF:
            VALUE = CFGF.get(KEY.name)
            setattr(cfg, KEY.name, Path(VALUE) if VALUE not in (None, "") else None)

    # Check the optional arguments that have not been set, to assign their default
    # values.
    # First check `information`.
    if cfg.information is None:
        cfg.information = DEFAULT_INFORMATION_PATH
        print_info(
            "`information` folder path unset: using the default `information` "
            "folder."
        )

    # Same check for `to_process`.
    if cfg.to_process is None:
        cfg.to_process = DEFAULT_TO_PROCESS_PATH
        print_info("`to_process` folder path unset: using the current path.")

    # Same check for `results`.
    if cfg.results is None:
        cfg.results = DEFAULT_RESULTS_PATH
        print_info("`results` folder path unset: using the current path.")

    return cfg.build()


def check_config_folders_correct(CFG: Config) -> None:
    """Check that the config folder paths are correct."""

    # Check that `information` folder is correct.
    if not CFG.information.is_dir():
        raise_error(
            f"`information` folder path: {CFG.information.resolve()} is not a "
            "directory"
        )

    # Check that `to_process` folder is correct.
    if not CFG.to_process.is_dir():
        raise_error(
            f"`to_process` folder path: {CFG.to_process.resolve()} is not a "
            "directory"
        )

    # Check that `results` folder is correct.
    if not CFG.results.is_dir():
        QUESTION = (
            format_error(
                f"`results` folder path: {CFG.results.resolve()} is not a directory.\n"
            )
            + "Do you want to create it?"
        )

        if form_yes_or_no(QUESTION):
            CFG.results.mkdir(parents=True)
            print_info("`results` folder created.")
        else:
            raise_error("You need to create a directory for the results.")


def main() -> None:
    cli(prog_name=NAME)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
radiocc structure data models.
"""

from enum import auto
from pathlib import Path
from typing import List, Optional, Union

from nptyping import NDArray
from pudb import set_trace as bp  # noqa

from radiocc.util import AutoEnum, GetAttributes, raise_error


class Folders(AutoEnum):
    """Folders"""

    information = auto()
    to_process = auto()
    results = auto()


class ResultsFolders(AutoEnum):
    """Results folders"""

    DATA = auto()
    PLOTS = auto()
    EPHEMERIDS = auto()


class FoldersError(AutoEnum):
    """Possible error related to the setting of the path to the folders."""

    NotADir = auto()
    MissingFolder = auto()


class Bands(AutoEnum):
    """Possible frequency bands."""

    S = auto()
    X = auto()
    Diff = auto()


class Layers(AutoEnum):
    """Possible atmospheric layers."""

    Atmo = auto()
    Iono = auto()


class ProcessType(AutoEnum):
    """Possible type of folder to be processed."""

    MEX = auto()
    MAVEN = auto()


class LV2Loops(AutoEnum):
    """Possible type of folder in LVL2 data loop."""

    IFMS = auto()
    DSN = auto()


class ConfigKeysReq(AutoEnum):
    """Required keys in config file."""


class ConfigKeysOpt(AutoEnum):
    """Optional keys in config file."""

    information = auto()
    to_process = auto()
    results = auto()
    folders = auto()


class Config(GetAttributes):
    """Structure representation the config."""

    def __init__(self, INFORMATION: Path, TO_PROCESS: Path, RESULTS: Path) -> None:
        self.information = INFORMATION
        self.to_process = TO_PROCESS
        self.results = RESULTS

    def __str__(self) -> str:
        return self.__repr__()

    def __repr__(self) -> str:
        return "\n".join(
            f"{attribute}: {value}"
            for (attribute, value) in self.get_attributes().items()
        )


class ConfigBuilder(GetAttributes):
    """Structure representation the config."""

    information: Optional[Path] = None
    to_process: Optional[Path] = None
    results: Optional[Path] = None

    def build(self) -> Config:
        for (attribute, value) in self.get_attributes().items():
            if value is None:
                raise_error(f"the config variable {attribute} could not be set.")

        return Config(
            self.information, self.to_process, self.results  # type: ignore [arg-type]
        )


class Scenario:
    """Structure representation of a scenario."""

    def __init__(
        self, TO_PROCESS: Path, BAND: Bands, LAYER: Layers, INDEX_PROCESS: int
    ) -> None:
        self.TO_PROCESS = TO_PROCESS
        self.BAND = BAND
        self.LAYER = LAYER
        self.INDEX_PROCESS = INDEX_PROCESS

    def results(self, RESULTS: Path) -> Path:
        return RESULTS / self.TO_PROCESS.name


class MexData:
    """Input data from MEX."""

    ET: NDArray[float]
    UTC: List[str]
    DISTANCE: NDArray[float]
    DOPPLER: NDArray[float]
    DIFF_DOPPLER: NDArray[float]
    ERROR: NDArray[float]
    FSUP: float
    SURFACES_CONDITIONS: NDArray[bool]
    INTEGRAL_CONDITIONS: NDArray[bool]


class MavenData:
    """Input data from MAVEN."""

    ET: NDArray[float]
    # FSKY: NDArray[float]
    DOPPLER: NDArray[float]


class L2Data:
    """Input data."""

    FOLDER_TYPE: ProcessType
    METADATA_FILE: Path
    DATA_FILE: Path
    dsn_station_naif_code: int
    DISTANCE_UNIT: str
    PLANET_NAIF_CODE: int
    SPACECRAFT_NAIF_CODE: int
    DATA: Union[MexData, MavenData]


class Export:
    """Structure representation of the data to be exported for a scenario."""

    DATA_PATH: Path
    PLOT_PATH: Path
    EPHE_PATH: Path
    DOPPLER: NDArray[float]
    DOPPLER_DEBIAS: NDArray[float]
    DOPPLER_BIAS_FIT: NDArray[float]
    DISTANCE: NDArray[float]
    REFRACTIVITY: NDArray[float]
    NE: NDArray[float]
    TEC: Optional[NDArray[float]]
    ERROR: NDArray[float]
    ERROR_REFRAC: NDArray[float]
    ERROR_ELEC: NDArray[float]



import numpy as np
import os
from scipy import signal as sg
from pudb import set_trace as bp  # noqa

from radiocc.model import ProcessType


# Type = "Quadratic" or "Linear", threshold [m]
def Off_Cor(
    Type,
    threshold,
    Doppler,
    Diff_doppler,
    ET,
    N_data,
    distance,
    CODE_DIR,
    i_integral,
    Threshold_Surface,
    Threshold_int,
    FOLDER_TYPE,
):

    Doppler_debias = np.full(N_data, np.nan)
    Doppler_biasfit = np.full(N_data, np.nan)

    # altitude threshold for fit
    Corr_ind = 0
    for i in range(len(distance)):
        if distance[i] > threshold:
            Corr_ind += 1
    if Corr_ind == len(distance):
        Corr_ind = Corr_ind - 1
    # Search for maximum

    if len(Doppler) % 2 == 1:
        taille = len(Doppler)
    else:
        taille = len(Doppler) - 1

    Smoothed_Doppler = sg.savgol_filter(Doppler, taille, 40)
    deriv = np.gradient(Smoothed_Doppler)

    good = np.logical_and(
        deriv[np.array(np.where(deriv[:-1] < 1e-5)) - 1] < 0,
        deriv[np.array(np.where(deriv[:-1] < 1e-5)) + 1] > 0,
    )

    targ = np.where(deriv[:-1] < 1e-5)
    index = targ[0][good[0]]
    good_one = index[-1]

    # FIT
    if Type == "Fit":
        p = np.polyfit(
            np.append(ET[:Corr_ind], ET[good_one]),
            np.append(Doppler[:Corr_ind], Doppler[good_one]),
            2,
        )

        for i in range(i_integral, N_data):
            Doppler_biasfit[i] = p[0] * ET[i] ** 2 + p[1] * ET[i] + p[2]
    ###########################################################################################

    delET = []
    for i in range(N_data):
        delET.append(ET[i])
    # FIXME:
    # checkout Doppler_biasfit computation ET vs delET

    # bias fit
    if Type == "Linear":
        p = np.polyfit(ET[i_integral:Corr_ind], Doppler[i_integral:Corr_ind], 1)
        p0 = p[0]
        p1 = p[1]
        for i in range(N_data):
            Doppler_biasfit[i] = p[0] * (ET[i]) + p[1]

    elif Type == "Quadratic":
        p = np.polyfit(ET[i_integral:Corr_ind], Doppler[i_integral:Corr_ind], 2)
        p0 = p[0]
        p1 = p[1]
        p2 = p[2]
        for i in range(N_data):
            Doppler_biasfit[i] = p[0] * (ET[i]) ** 2 + p[1] * (ET[i] - ET[0]) + p[2]

    else:
        print("Type is not recognized")

    # subtract bias and scale residual
    for i in range(N_data):
        Doppler_debias[i] = Doppler[i] - Doppler_biasfit[i]

    if Type == "Linear":
        return (
            i_integral,
            ET,
            delET,
            Doppler_debias,
            Doppler_biasfit,
            Corr_ind,
            p0,
            p1,
        )

    if Type == "Quadraticuadratic":
        return (
            i_integral,
            ET,
            delET,
            Doppler_debias,
            Doppler_biasfit,
            Corr_ind,
            p0,
            p1,
            p2,
        )



#!/usr/bin/env python3

"""
Export the graphs and text output.
"""

from typing import Optional  # noqa

import numpy
from pudb import set_trace as bp  # noqa

from radiocc import Export, L2Data, Scenario


def run(SCENARIO: Scenario, L2DATA: L2Data, OUT: Export) -> None:
    """Export results."""

    PATH_FILTER = f"{SCENARIO.BAND.name}-{SCENARIO.LAYER.name.upper()}-%d.txt"
    COUNT = len(list(OUT.DATA_PATH.glob(PATH_FILTER)))
    PATH_OUT = OUT.DATA_PATH / (PATH_FILTER % (COUNT + 1))

    output = numpy.stack(
        (
            OUT.DOPPLER,
            OUT.DOPPLER_DEBIAS,
            OUT.DOPPLER_BIAS_FIT,
            OUT.DISTANCE,
            OUT.REFRACTIVITY,
            OUT.NE,
            OUT.ERROR,
            OUT.ERROR_REFRAC,
            OUT.ERROR_ELEC,
        ),
        axis=1,
    )

    if OUT.TEC is not None:
        output = numpy.concatenate((output, OUT.TEC[:, None]), axis=1)

    numpy.savetxt(
        PATH_OUT,
        output,
        fmt="%.18e",
        delimiter=" ",
        newline="\n",
        header="",
        footer="",
        comments="# ",
    )



#!/usr/bin/env python3
# mypy: ignore-errors

"""
Reconstruct.
"""

from pathlib import Path
from typing import Any  # noqa

import numpy
from pudb import set_trace as bp  # noqa

from radiocc import Config, constants, old
from radiocc.model import Export, L2Data, ProcessType, ResultsFolders, Scenario
from radiocc.old import (
    R4_State_Vectors_twoway_v2,
    R5_Foot_Print,
    R6_Framework_Conversion,
    R7_Offset_Correction,
    R8_Plot_Doppler_1,
    R15_Errors,
    R16_Biasfit_Correction,
    R17_Run_Routines1,
    R18_Occ_Info_MVN,
)


def run(  # noqa: C901
    cfg: Config,
    scenario: Scenario,
    l2_data: L2Data,
) -> Export:
    # =================================================================================
    # Interface between new & old code
    # =================================================================================
    DATA = l2_data.DATA
    ET = DATA.ET
    N_data = ET.shape[0]
    Doppler = DATA.DOPPLER

    if l2_data.FOLDER_TYPE == ProcessType.MEX:
        Diff_doppler = DATA.DIFF_DOPPLER
        distance = DATA.DISTANCE
        i_Surface = numpy.where(DATA.SURFACES_CONDITIONS)[0]
        i_integral = numpy.where(DATA.INTEGRAL_CONDITIONS)[0]
        i_Surface = N_data - 1 if not i_Surface.size else i_Surface[0]
        i_integral = 0 if not i_Surface.size else i_integral[0]
        fsup = DATA.FSUP
        error = DATA.ERROR
    else:
        # fsky = DATA.FSKY
        error = numpy.zeros(N_data)
        error_ref = numpy.zeros(N_data)
        error_elec = numpy.zeros(N_data)
        error_neut = numpy.zeros(N_data)
        Diff_doppler = numpy.zeros(N_data)
        fsup = 7.2e9
        i_integral = None

    NAIF_SC = str(l2_data.SPACECRAFT_NAIF_CODE)
    NAIF_GS = str(l2_data.dsn_station_naif_code)
    NAIF_P = str(l2_data.PLANET_NAIF_CODE)

    item = scenario.LAYER.name.upper()
    Bande = scenario.BAND.name.upper()
    RESULTS = scenario.results(cfg.results)
    DATA_PRO = str(scenario.TO_PROCESS.parent)
    DATA_ID_i_profile = str(scenario.TO_PROCESS.resolve())
    CODE_DIR = str(Path(old.__file__).parent.resolve())
    DATA_DIR = str(scenario.TO_PROCESS.parent.parent.resolve())
    PLOT_DIR = str((RESULTS / ResultsFolders.PLOTS.name).resolve())
    EPHE_DIR = str((RESULTS / ResultsFolders.EPHEMERIDS.name).resolve())
    DATA_FINAL_DIR = str((RESULTS / ResultsFolders.DATA.name).resolve())

    c = constants.c
    R_Planet = constants.R_Planet
    e = constants.e
    eps0 = constants.eps0
    me = constants.me
    CC = constants.CC
    kB = constants.kB
    V = constants.V
    Ref_frame = constants.Ref_frame
    B_Cor_Type = constants.B_Cor_Type
    Threshold_Cor = constants.Threshold_Cor
    Threshold_int = constants.Threshold_int
    Threshold_neut_upp = constants.Threshold_neut_upp
    Threshold_Surface = constants.Threshold_Surface
    # =================================================================================

    # R5: State_Vectors_twoway
    (
        t0_up,
        pos_GS_up,
        vel_GS_up,
        pos_MEX_up,
        vel_MEX_up,
        tMEX,
        t0_dn,
        pos_GS_dn,
        vel_GS_dn,
        pos_MEX_dn,
        vel_MEX_dn,
        t0_upp,
        pos_GS_upp,
        vel_GS_upp,
        pos_MEX_upp,
        vel_MEX_upp,
        pos_MEX,
        vel_MEX,
        ET_MEX_Mars,
    ) = R4_State_Vectors_twoway_v2.ephemerides(
        N_data, ET, NAIF_SC, NAIF_GS, NAIF_P, DATA_PRO, Ref_frame, l2_data.FOLDER_TYPE
    )

    # R6: footprint
    d_lat, d_lon, dist_Mars_Sun, spclon_SC, spclat_SC, re, f = R5_Foot_Print.long_lat(
        N_data, pos_MEX, vel_MEX, ET, ET_MEX_Mars, Ref_frame, l2_data.FOLDER_TYPE
    )

    # R7 coordinate frames
    (
        r_MEX_up,
        z_MEX_up,
        z_GS_up,
        vr_MEX_up,
        vz_MEX_up,
        vr_GS_up,
        vz_GS_up,
        gamma_up,
        beta_e_up,
        delta_s_up,
    ) = R6_Framework_Conversion.Occ_Plane_Conversion(
        N_data, pos_MEX_up, vel_MEX_up, pos_GS_up, vel_GS_up
    )

    # R7 coordinate frames
    (
        r_MEX_dn,
        z_MEX_dn,
        z_GS_dn,
        vr_MEX_dn,
        vz_MEX_dn,
        vr_GS_dn,
        vz_GS_dn,
        gamma_dn,
        beta_e_dn,
        delta_s_dn,
    ) = R6_Framework_Conversion.Occ_Plane_Conversion(
        N_data, pos_MEX_dn, vel_MEX_dn, pos_GS_dn, vel_GS_dn
    )

    if l2_data.FOLDER_TYPE == ProcessType.MAVEN:
        # R18 : Occ_info :
        distance = R18_Occ_Info_MVN.occultation_info(
            NAIF_P,
            NAIF_GS,
            ET,
            Ref_frame,
            NAIF_SC,
            DATA_PRO,
            DATA_ID_i_profile,
            Threshold_Surface,
            EPHE_DIR,
        )

        i_Surface = numpy.where(numpy.array(distance) <= Threshold_Surface)[0]
        if i_Surface.size == 0:
            i_Surface = N_data - 1
        else:
            i_Surface = i_Surface[0]
        i_integral = numpy.where(numpy.array(distance) <= Threshold_int)[0]
        if i_integral.size == 0:
            i_integral = 0
        else:
            i_integral = i_integral[0]

    if B_Cor_Type == "Linear":
        (
            i_integral,
            ET,
            delET,
            Doppler_debias,
            Doppler_biasfit,
            Corr_ind,
            p0,
            p1,
        ) = R7_Offset_Correction.Off_Cor(
            B_Cor_Type,
            Threshold_Cor,
            Doppler,
            Diff_doppler,
            ET,
            N_data,
            distance,
            CODE_DIR,
            i_integral,
            Threshold_Surface,
            Threshold_int,
            l2_data.FOLDER_TYPE,
        )
    if B_Cor_Type == "Quadratic":
        (
            i_integral,
            i_Surface,
            ET,
            delET,
            Doppler_debias,
            Doppler_biasfit,
            Corr_ind,
            p0,
            p1,
            p2,
        ) = R7_Offset_Correction.Off_Cor(
            i_integral,
            i_Surface,
            ET,
            delET,
            B_Cor_Type,
            Threshold_Cor,
            Doppler,
            Diff_doppler,
            ET,
            N_data,
            distance,
            CODE_DIR,
            i_integral,
            Threshold_Surface,
            Threshold_int,
            l2_data.FOLDER_TYPE,
        )

    # R5: Plot doppler after bias correction
    R8_Plot_Doppler_1.PLOT_Dop1(
        distance, Doppler, Doppler_debias, Doppler_biasfit, ET, PLOT_DIR, N_data
    )
    HAPPY_or_NOT = str(input("Enter 'YES' if the debias is good and 'NO' if bad: "))
    while HAPPY_or_NOT == "NO":
        NEW_THRESHOLD = float(input("Enter new threshold altitude = "))
        NEW_TYPE = str(input("Enter new type, Linear or Quadratic: "))
        if NEW_TYPE == "Linear":
            (
                i_integral,
                ET,
                delET,
                Doppler_debias,
                Doppler_biasfit,
                Corr_ind,
                p0,
                p1,
            ) = R7_Offset_Correction.Off_Cor(
                NEW_TYPE,
                NEW_THRESHOLD,
                Doppler,
                Diff_doppler,
                ET,
                N_data,
                distance,
                CODE_DIR,
                i_integral,
                Threshold_Surface,
                Threshold_int,
                l2_data.FOLDER_TYPE,
            )
        if NEW_TYPE == "Quadratic":
            (
                i_integral,
                ET,
                delET,
                Doppler_debias,
                Doppler_biasfit,
                Corr_ind,
                p0,
                p1,
                p2,
            ) = R7_Offset_Correction.Off_Cor(
                NEW_TYPE,
                NEW_THRESHOLD,
                Doppler,
                Diff_doppler,
                ET,
                N_data,
                distance,
                CODE_DIR,
                i_integral,
                Threshold_Surface,
                Threshold_int,
                l2_data.FOLDER_TYPE,
            )

        R8_Plot_Doppler_1.PLOT_Dop1(
            distance, Doppler, Doppler_debias, Doppler_biasfit, ET, PLOT_DIR, N_data
        )
        HAPPY_or_NOT = str(input("Enter 'YES' if the debias is good and 'NO' if bad: "))
        if HAPPY_or_NOT == "YES":
            break
    if item == "IONO":
        refractivity, Ne, TEC = R17_Run_Routines1.run(
            Doppler_debias,
            N_data,
            r_MEX_up,
            z_MEX_up,
            z_GS_up,
            vr_MEX_up,
            vz_MEX_up,
            vr_GS_up,
            vz_GS_up,
            gamma_up,
            beta_e_up,
            delta_s_up,
            r_MEX_dn,
            z_MEX_dn,
            z_GS_dn,
            vr_MEX_dn,
            vz_MEX_dn,
            vr_GS_dn,
            vz_GS_dn,
            gamma_dn,
            beta_e_dn,
            delta_s_dn,
            Bande,
            fsup,
            c,
            Diff_doppler,
            item,
            i_integral,
            i_Surface,
            R_Planet,
            e,
            eps0,
            me,
            distance,
            Doppler,
            PLOT_DIR,
            Threshold_neut_upp,
            CC,
            kB,
            EPHE_DIR,
            NAIF_SC,
            NAIF_P,
            NAIF_GS,
            ET,
            Ref_frame,
            DATA_ID_i_profile,
            DATA_DIR,
            Threshold_Surface,
        )
    if item == "ATMO":
        refractivity, Nn = R17_Run_Routines1.run(
            Doppler_debias,
            N_data,
            r_MEX_up,
            z_MEX_up,
            z_GS_up,
            vr_MEX_up,
            vz_MEX_up,
            vr_GS_up,
            vz_GS_up,
            gamma_up,
            beta_e_up,
            delta_s_up,
            r_MEX_dn,
            z_MEX_dn,
            z_GS_dn,
            vr_MEX_dn,
            vz_MEX_dn,
            vr_GS_dn,
            vz_GS_dn,
            gamma_dn,
            beta_e_dn,
            delta_s_dn,
            Bande,
            fsup,
            c,
            Diff_doppler,
            item,
            i_integral,
            i_Surface,
            R_Planet,
            e,
            eps0,
            me,
            distance,
            Doppler,
            PLOT_DIR,
            Threshold_neut_upp,
            CC,
            kB,
            EPHE_DIR,
            NAIF_SC,
            NAIF_P,
            NAIF_GS,
            ET,
            Ref_frame,
            DATA_ID_i_profile,
            DATA_DIR,
            Threshold_Surface,
        )

    RefCorr_GD_or_NOT = str(
        input("Enter 'YES' if the refractivity correction is good and 'NO' if bad: ")
    )
    while RefCorr_GD_or_NOT == "NO":
        if B_Cor_Type == "Linear":
            print("\t p0 = ", p0)
            print("\t p1 = ", p1)

        if B_Cor_Type == "Quadratic":
            print("\t p0 = ", p0)
            print("\t p1 = ", p1)
            print("\t p2 = ", p2)
        R16_Biasfit_Correction.Off_Cor2(
            B_Cor_Type, Doppler, Diff_doppler, ET, N_data, distance
        )
        if item == "IONO":
            refractivity, Ne, TEC = R17_Run_Routines1.run(
                Doppler_debias,
                N_data,
                r_MEX_up,
                z_MEX_up,
                z_GS_up,
                vr_MEX_up,
                vz_MEX_up,
                vr_GS_up,
                vz_GS_up,
                gamma_up,
                beta_e_up,
                delta_s_up,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Bande,
                fsup,
                c,
                Diff_doppler,
                item,
                i_integral,
                i_Surface,
                R_Planet,
                e,
                eps0,
                me,
                distance,
                Doppler,
                PLOT_DIR,
                Threshold_neut_upp,
                CC,
                kB,
                EPHE_DIR,
                NAIF_SC,
                NAIF_P,
                NAIF_GS,
                ET,
                Ref_frame,
                DATA_ID_i_profile,
                DATA_DIR,
                Threshold_Surface,
            )
        if item == "ATMO":
            refractivity, Nn = R17_Run_Routines1.run(
                Doppler_debias,
                N_data,
                r_MEX_up,
                z_MEX_up,
                z_GS_up,
                vr_MEX_up,
                vz_MEX_up,
                vr_GS_up,
                vz_GS_up,
                gamma_up,
                beta_e_up,
                delta_s_up,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Bande,
                fsup,
                c,
                Diff_doppler,
                item,
                i_integral,
                i_Surface,
                R_Planet,
                e,
                eps0,
                me,
                distance,
                Doppler,
                PLOT_DIR,
                Threshold_neut_upp,
                CC,
                kB,
                EPHE_DIR,
                NAIF_SC,
                NAIF_P,
                NAIF_GS,
                ET,
                Ref_frame,
                DATA_ID_i_profile,
                DATA_DIR,
                Threshold_Surface,
            )

        RefCorr_GD_or_NOT = str(
            input(
                "Enter 'YES' if the refractivity correction is good and 'NO' if bad: "
            )
        )

        if RefCorr_GD_or_NOT == "YES":
            break
    error_ref, error_elec, error_neut = R15_Errors.ERRORS(
        N_data, fsup, error, c, distance, Threshold_neut_upp, V, e, eps0, me, Bande, CC
    )

    # Return variables to be exported.

    export = Export()
    export.DATA_PATH = Path(DATA_FINAL_DIR)
    export.PLOT_PATH = Path(PLOT_DIR)
    export.EPHE_PATH = Path(EPHE_DIR)
    export.DOPPLER = Doppler
    export.DOPPLER_DEBIAS = Doppler_debias
    export.DOPPLER_BIAS_FIT = Doppler_biasfit
    export.DISTANCE = distance
    export.REFRACTIVITY = refractivity
    export.ERROR = error
    export.ERROR_REFRAC = error_ref

    if item == "IONO":
        export.NE = Ne
        export.ERROR_ELEC = error_elec
        export.TEC = TEC
    else:
        export.NE = Nn
        export.ERROR_ELEC = error_neut
        export.TEC = None

    return export



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Sep  1 12:22:41 2020

@author: ananya
"""
import R8_Plot_Doppler_1
import R9_BendAng_ImpParam_up
import R9_BendAng_ImpParam_dn
import R10_Avg_BendAng_ImpParam
import R11_Refractivity_and_Bending_Radius_v2
import R12_Electron_Density
import R13_Neutral_Number_Density
import R14_Plot_check
import R16_Biasfit_Correction

def run(Doppler_debias,N_data, r_MEX_up, z_MEX_up, z_GS_up, vr_MEX_up, vz_MEX_up, vr_GS_up, vz_GS_up, gamma_up, beta_e_up,ET, \
        delta_s_up, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, delta_s_dn, B_Cor_Type, \
        Bande,fsup,c, Diff_doppler, item,i_integral, i_Surface, R_Planet,e, eps0, me, distance,Doppler, PLOT_DIR, Threshold_neut_upp,CC, kB):

    R16_Biasfit_Correction.Off_Cor2(B_Cor_Type, Doppler, Diff_doppler, ET, N_data, distance)
    R8_Plot_Doppler_1.PLOT_Dop1(distance,Doppler,Doppler_debias,PLOT_DIR)
    HAPPY_or_NOT = str(input("Enter 'YES' if the debias is good and 'NO' if bad: " ))
    if HAPPY_or_NOT == 'YES':
            kx = 880./749.
            ks = 240./749.
            Doppler_debias_dn_iono_x = Doppler_debias/(1.0+(kx*kx))
            Doppler_debias_up_iono_x = Doppler_debias/(kx+(1.0/kx))

            Doppler_debias_dn_iono_s = Doppler_debias/(1.0+(ks**2))
            Doppler_debias_up_iono_s = Doppler_debias_up_iono_x

            Doppler_debias_dn_atmo_x = Doppler_debias/2.0
            Doppler_debias_up_atmo_x = Doppler_debias/(2.0*kx)

            Doppler_debias_dn_atmo_s = Doppler_debias/2.0
            Doppler_debias_up_atmo_s = Doppler_debias_up_atmo_x

            Diff_Doppler_debias_dn_iono_s = Diff_doppler/(1.-(9./121.))

            if Bande == 'X':

                if item == 'IONO' :


                    # R8: bending angle & impact parameter up
                    imp_param_up, bend_ang_up, delta_r_up, beta_r_up \
                        = R9_BendAng_ImpParam_up.Bend_up(N_data, r_MEX_up, z_MEX_up, z_GS_up, vr_MEX_up, \
                                                   vz_MEX_up, vr_GS_up, vz_GS_up, gamma_up, beta_e_up, \
                                                   delta_s_up, Doppler_debias_up_iono_x,fsup,c)

                    # R8: bending angle & impact parameter down
                    imp_param_dn, bend_ang_dn, delta_r_dn, beta_r_dn \
                        = R9_BendAng_ImpParam_dn.Bend_dn(N_data, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, \
                                                   vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, \
                                                   delta_s_dn, Doppler_debias_dn_iono_x,Bande,fsup,c)

                    # R9: Average bending angle & impact parameter
                    imp_param, bend_ang \
                        = R10_Avg_BendAng_ImpParam.Avg(imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn,Bande)


                if item == 'ATMO' :
                    # R8: bending angle & impact parameter up
                    imp_param_up, bend_ang_up, delta_r_up, beta_r_up \
                        = R9_BendAng_ImpParam_up.Bend_up(N_data, r_MEX_up, z_MEX_up, z_GS_up, vr_MEX_up, \
                                                   vz_MEX_up, vr_GS_up, vz_GS_up, gamma_up, beta_e_up, \
                                                   delta_s_up, Doppler_debias_up_atmo_x,fsup,c)

                    # R8: bending angle & impact parameter down
                    imp_param_dn, bend_ang_dn, delta_r_dn, beta_r_dn \
                        = R9_BendAng_ImpParam_dn.Bend_dn(N_data, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, \
                                                   vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, \
                                                   delta_s_dn, Doppler_debias_dn_atmo_x,Bande,fsup,c)

                    # R9: Average bending angle & impact parameter
                    imp_param, bend_ang \
                        = R10_Avg_BendAng_ImpParam.Avg(imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn,Bande)


            if Bande == 'S':
                if item == 'IONO' :
                    # R8: bending angle & impact parameter up
                    imp_param_up, bend_ang_up, delta_r_up, beta_r_up \
                        = R9_BendAng_ImpParam_up.Bend_up(N_data, r_MEX_up, z_MEX_up, z_GS_up, vr_MEX_up, \
                                                   vz_MEX_up, vr_GS_up, vz_GS_up, gamma_up, beta_e_up, \
                                                   delta_s_up, Doppler_debias_up_iono_s,fsup,c)

                    # R8: bending angle & impact parameter down
                    imp_param_dn, bend_ang_dn, delta_r_dn, beta_r_dn \
                        = R9_BendAng_ImpParam_dn.Bend_dn(N_data, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, \
                                                   vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, \
                                                   delta_s_dn, Doppler_debias_dn_iono_s,Bande,fsup,c)

                    # R9: Average bending angle & impact parameter
                    imp_param, bend_ang \
                        = R10_Avg_BendAng_ImpParam.Avg(imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn,Bande)


                if item == 'ATMO' :
                    # R8: bending angle & impact parameter up
                    imp_param_up, bend_ang_up, delta_r_up, beta_r_up \
                        = R9_BendAng_ImpParam_up.Bend_up(N_data, r_MEX_up, z_MEX_up, z_GS_up, vr_MEX_up, \
                                                   vz_MEX_up, vr_GS_up, vz_GS_up, gamma_up, beta_e_up, \
                                                   delta_s_up, Doppler_debias_up_atmo_s,fsup,c)

                    # R8: bending angle & impact parameter down
                    imp_param_dn, bend_ang_dn, delta_r_dn, beta_r_dn \
                        = R9_BendAng_ImpParam_dn.Bend_dn(N_data, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, \
                                                   vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, \
                                                   delta_s_dn, Doppler_debias_dn_atmo_s,Bande,fsup,c)

                    # R9: Average bending angle & impact parameter
                    imp_param, bend_ang \
                        = R10_Avg_BendAng_ImpParam.Avg(imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn,Bande)

            if Bande == 'Diff':


                if item == 'IONO' :


                    # R8: bending angle & impact parameter down
                    imp_param, bend_ang, delta_r_diff, beta_r_diff \
                        = R9_BendAng_ImpParam_dn.Bend_dn(N_data, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, \
                                                   vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, \
                                                   delta_s_dn, Diff_Doppler_debias_dn_iono_s,Bande,fsup,c)


                if item == 'ATMO' :

                    # R8: bending angle & impact parameter down
                    imp_param, bend_ang, delta_r_dn, beta_r_dn \
                        = R9_BendAng_ImpParam_dn.Bend_dn(N_data, r_MEX_dn, z_MEX_dn, z_GS_dn, vr_MEX_dn, \
                                                   vz_MEX_dn, vr_GS_dn, vz_GS_dn, gamma_dn, beta_e_dn, \
                                                   delta_s_dn, Doppler_debias_dn_atmo_s,Bande,fsup,c)   #NEED TO CHECK????

            # R11
            ref_index,refractivity, bend_radius, Sum_tot   =                          \
                R11_Refractivity_and_Bending_Radius_v2.Abel_Analytical(N_data, i_integral, i_Surface,\
                                                                   imp_param, bend_ang, R_Planet)

            if item == 'IONO' :

                Ne  = R12_Electron_Density.Elec(N_data, refractivity, e, eps0, me,Bande,fsup,c)
                TEC = R12_Electron_Density.TEC_Calc(N_data, Ne, i_integral, i_Surface,      \
                                                   bend_radius, R_Planet)

                R14_Plot_check.PLOT1(distance,Doppler,Doppler_debias,refractivity, Ne,PLOT_DIR,item)
            if item == 'ATMO' :

                # R13: neutral density (Level 04)
                Nn, i_neut_upp =                                                          \
                    R13_Neutral_Number_Density.Neut(N_data, bend_radius, refractivity,    \
                                                    Threshold_neut_upp, i_Surface, CC, kB)
                R14_Plot_check.PLOT2(distance,Doppler,Doppler_debias,refractivity, Nn,PLOT_DIR,item)

    return



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Sep  1 07:32:42 2020

@author: ananya
"""

import numpy as np
def Off_Cor2(Type, Doppler, Diff_doppler, ET, N_data, distance):
    Doppler_debias = np.full(N_data, np.nan)
    Doppler_biasfit = np.full(N_data, np.nan)



    if Type == 'Linear':
        #p = np.polyfit(ET[i_integral:Corr_ind],Doppler[i_integral:Corr_ind],1)
        p0 = float(input("Enter the new p[0], i.e; p[0] +dp[0] = " ))
        p1 = float(input("Enter the new p[1], i.e; p[1] +dp[1] = " ))
        for i in range(N_data):
            Doppler_biasfit[i] = p0 * (ET[i]) + p1

    elif Type == 'Quadratic':
        #p = np.polyfit(ET[i_integral:Corr_ind],Doppler[i_integral:Corr_ind],2)
        p0 = float(input("Enter the new p[0], i.e; p[0] +dp[0] = " ))
        p1 = float(input("Enter the new p[1], i.e; p[1] +dp[1] = " ))
        p2 = float(input("Enter the new p[2], i.e; p[2] +dp[2] = " ))
        for i in range(N_data):
            Doppler_biasfit[i] = p0 * (ET[i])**2 + p1*(ET[i]-ET[0]) + p2
    else:
        print('Type is not recognized')

    for i in range(N_data):
        Doppler_debias[i]  = (Doppler[i]- Doppler_biasfit[i])

    return Doppler_debias, Doppler_biasfit



#!/usr/bin/env python3

"""
Radio occultation.
"""

from pathlib import Path

from pudb import set_trace as bp  # noqa: F401

from radiocc import (
    Bands,
    Config,
    Layers,
    ResultsFolders,
    Scenario,
    constants,
    export,
    old,
    process_parser,
    reconstruct,
)
from radiocc.old import R17_Plot_profiles

BANDS = (Bands.X,)
LAYERS = (Layers.Iono, Layers.Atmo)


def initialization(CFG: Config) -> None:
    """Initialization"""

    for INDEX_PROCESS, PROCESS_PATH in enumerate(CFG.to_process.iterdir()):
        print(f"Reading {PROCESS_PATH.name}..")

        for BAND in BANDS:
            print(f"  Band: {BAND.name}")

            for LAYER in LAYERS:
                print(f"    Layer: {LAYER.name}")

                SCENARIO = Scenario(PROCESS_PATH, BAND, LAYER, INDEX_PROCESS)

                run(CFG, SCENARIO)

            band_export(CFG, SCENARIO)


def run(CFG: Config, SCENARIO: Scenario) -> None:
    """Run a scenario."""
    process_parser.prepare_directories(CFG, SCENARIO)
    FOLDER_TYPE = process_parser.detect_folder_type(SCENARIO.TO_PROCESS)

    if FOLDER_TYPE is None:
        return None

    process_parser.load_spice_kernels(SCENARIO.TO_PROCESS, FOLDER_TYPE)
    L2_DATA = process_parser.read_L2_data(CFG, SCENARIO, FOLDER_TYPE)

    if L2_DATA is None:
        return None

    EXPORT = reconstruct.run(CFG, SCENARIO, L2_DATA)

    export.run(SCENARIO, L2_DATA, EXPORT)


def band_export(CFG: Config, SCENARIO: Scenario) -> None:
    """Finish the plots for the layers of a scenario."""
    # Interface with old code.
    RESULTS = SCENARIO.results(CFG.results)
    i_Profile = SCENARIO.INDEX_PROCESS
    DATA_PRO = str(SCENARIO.TO_PROCESS.parent)
    DATA_ID = str(SCENARIO.TO_PROCESS.resolve())
    CODE_DIR = str(Path(old.__file__).parent.resolve())
    DATA_DIR = str(SCENARIO.TO_PROCESS.parent.parent.resolve())
    PLOT_DIR = str((RESULTS / ResultsFolders.PLOTS.name).resolve())
    DATA_FINAL_DIR = str((RESULTS / ResultsFolders.DATA.name).resolve())

    R17_Plot_profiles.PLOT2(  # type: ignore [no-untyped-call]
        DATA_ID,
        i_Profile,
        DATA_PRO,
        CODE_DIR,
        SCENARIO.BAND.name,
        DATA_DIR,
        constants.Threshold_Cor,
        constants.Threshold_Surface,
        DATA_FINAL_DIR,
        PLOT_DIR,
        constants.Threshold_int,
    )



import numpy as np
#import matplotlib.pyplot as plt
def Neut(N_data, bend_radius, refractivity, d_neut_upp, i_Surface, C1, kB):

    i_neut_upp = np.where(bend_radius >= d_neut_upp)[0][-2]
    Nn = np.array([np.nan for i in range(N_data)], float)

    for i in range(i_neut_upp,i_Surface):

        Nn[i] = refractivity[i]/C1*1E-6
    #print(refractivity, Nn)
    #print('')
    print('\t NEUTRAL DENSITY DONE')
    #print('')
    #print('')

    return Nn, i_neut_upp



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Nov  8 10:46:27 2019

@author: ananya
"""

import numpy as np

def Elec(N_data, refractivity, e, eps0, me,Bande,fsup,c):

    if Bande == 'X':
        f_TX=(fsup)*(880./749.)  # downward X-band
    if Bande == 'S':
        f_TX=(fsup)*(240./749.)  # downward S-band
    if Bande == 'Diff':
        f_TX=(fsup)*(240./749.)  # downward S-band


    Ne = np.full(N_data,np.nan)#np.array([np.nan for i in range( N_data )], float)

    for i in range(N_data):

        Ne[i] = - (refractivity[i]*1E-6)/40.31*f_TX**2
        #Ne_dn[i] = -2*np.pi*(refractivity[i]*1E-6)/(2.818E-15*(c/f_TX)**2)

        #Ne_dn[i] = - refractivity[i]*(8*np.pi**2*me*eps0*f_TX**2)/(e**2)*1E-6
    #print(f_TX)
    #print(Ne[1600])
    return Ne


def TEC_Calc(N_data, Ne, Corr_ind, i_Surface, bend_radius, R_Mars):

    TEC = np.full(N_data,np.nan)#np.array([np.nan for i in range( N_data )], float)
    TEC_loc = 0.
    for i in range(Corr_ind+1, i_Surface):

        if Ne[i] > 0:

            TEC_loc += Ne[i] * ((bend_radius[i-1]-bend_radius[i])) / 1.0e16
            TEC[i] = TEC_loc

    #print np.shape(TEC)
    #print('')
    #print('')
    print('\t ELECTRON DENSITY DONE')
    #print('')
    #print('')

    return TEC



# -*- coding: utf-8 -*-
"""
Created on Wed Feb 20 11:43:18 2019

@author: akrishnan
"""




def Avg(imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn,Bande):

    #imp_param = (imp_param_up + imp_param_dn)/2
    #bend_ang  = (bend_ang_up + bend_ang_dn)/2

    imp_param = imp_param_dn
    bend_ang  = bend_ang_dn
    #print('\tR9: Done')
    return imp_param,bend_ang



import numpy as np
import scipy as sc
import spiceypy as spy

from radiocc.model import ProcessType


#   Variable  I/O  Description
#   --------  ---  --------------------------------------------------
#   method     I   Computation method.
#   target     I   Name of target body.
#   et         I   Epoch in TDB seconds past J2000 TDB.
#   fixref     I   Body-fixed, body-centered target body frame.
#   abcorr     I   Aberration correction flag.
#   obsrvr     I   Name of observing body.
#   spoint     O   Sub-observer point on the target body.
#   trgepc     O   Sub-observer point epoch.
#   srfvec     O   Vector from observer to sub-observer point.




def long_lat(N_data, pos_MEX, vel_MEX, ET, ET_MEX_Mars,Ref_Frame, FOLDER_TYPE: ProcessType):

##    	    Look up the target body's radii. We'll use these to
##          convert Cartesian to planetographic coordinates. Use
##          the radii to compute the flattening coefficient of
##          the reference ellipsoid.

    radii = spy.bodvrd('Mars', 'RADII', 3)

#Let 're' and 'rp' be, respectively, the equatorial and polar radii of the target.

    re = radii[1][0]

    rp = radii[1][2]

    f = (re-rp)/re

    odist     = np.zeros((N_data,))

    xabsSC    = np.zeros((N_data,))

    spglat_SC = np.zeros((N_data,))

    spglon_SC = np.zeros((N_data,))

    spgalt_SC = np.zeros((N_data,))

    spcrad_SC = np.zeros((N_data,))

    spclon_SC = np.zeros((N_data,))

    spclat_SC = np.zeros((N_data,))

    #Compute sub-observer point using light time (LT) and stellar (S) aberration corrections. Using ellipsoid
    #shape models,"intercept" sub-observer point definitions.

    for i in range(N_data):

        [spoint, trgepc, srfvec] = spy.subpnt('Intercept: ellipsoid', 'Mars', ET_MEX_Mars[i], 'IAU_MARS', 'LT+S', FOLDER_TYPE.name)

	# For target shapes modeled by ellipsoids, the
        # sub-observer point is defined either as the point on
        # the target body that is closest to the observer, or
        # the target surface intercept of the line from the
        # observer to the target's center. The components of `spoint' have units of km and are in Cartesian coordinates.
	#  'trgepc' is expressed as seconds past J2000 TDB.

        odist[i] = np.linalg.norm(srfvec) #Compute the MEX's distance from the "spoint" vector. Using the Frobenius norm.

        xabsSC[i] = np.linalg.norm(spoint) # Compute the distance from the center of Mars of the spoint.

	#Convert the sub-observer point's Cartesian coordinates to planetographic longitude, latitude and altitude.

        [spglon, spglat, spgalt] = spy.recpgr('Mars', spoint, re, f)

        spglon *= 180./sc.pi #Convert radians to degrees.

        spglat *= 180./sc.pi #Convert radians to degrees.

        spglon_SC[i] = spglon

        spglat_SC[i] = spglat

        spgalt_SC[i] = spgalt

	# Convert sub-observer point's Cartesian coordinates to planetocentric radius, longitude, and latitude.
        [spcrad, spclon, spclat] = spy.reclat(spoint)

        spclon *= 180./sc.pi #Convert radians to degrees.

        spclat *= 180./sc.pi #Convert radians to degrees.

        spcrad_SC[i] = spcrad

        spclon_SC[i] = spclon

        spclat_SC[i] = spclat


    pos_MEX_Earth = np.full((N_data,3),np.nan)#np.array([[np.nan for j in range(3)] for i in range( N_data )], dtype = float)

    for i in range(N_data):

	#Return the state (position and velocity) of MEX relative to EARTH, optionally corrected for light time (planetary aberration) and stellar  aberration.

	#[position, lt ] = spy.spkezr(target, time, reference_frame, aberration_correction, observing_body)
        [ position, lt ] = spy.spkezr( FOLDER_TYPE.name, ET_MEX_Mars[i], 'IAU_MARS', 'none', 'EARTH' )

	#lt is the one-way light time between the observer and target in seconds.

        pos_MEX_Earth[i] = position[:3]

	# Take the 3 first components of position corresponding to the -x -y and -z of the target's position; the last three components form the corresponding velocity vector. They are expressed with respect to the MARS's frame.

    ray_MEX_Earth = np.full((N_data,3),np.nan)#np.array([[np.nan for j in range(3)] for i in range( N_data )], dtype = float)

    for i in range(N_data):

        ray_MEX_Earth[i] = -pos_MEX_Earth[i]/np.linalg.norm(pos_MEX_Earth)

    A = radii[1][0] # Length of ellipsoid’s semi-axis in the x direction

    B = radii[1][1] # Length of ellipsoid’s semi-axis in the y direction

    C = radii[1][2] # Length of ellipsoid’s semi-axis in the z direction

    pnear_ray = np.full((N_data,3),np.nan)#np.array([[np.nan for j in range(3)] for i in range( N_data )], dtype = float)

    dist_ray = np.zeros((N_data,))


    for i in range(N_data):

        LINEPT = pos_MEX[i] *1e-3

        LINEDR = ray_MEX_Earth[i]


        [pnear, dist] = spy.npedln(A, B, C, LINEPT, LINEDR)

        pnear_ray[i] = pnear # point on the ellipsoid that is closest to the line if the line doesn't intersect the ellipsoid.

        dist_ray[i] = dist # distance of the line from the ellipsoid. Minimum distance between any point on the line and any point on the ellipsoid

    d_lon    = np.zeros((N_data,))

    d_lat    = np.zeros((N_data,))

    d_radius = np.zeros((N_data,))

    local_solar_time = []

    SZA = np.zeros((N_data,))

    d_phase = np.zeros((N_data,))

    for i in range(N_data):

	# Convert from rectangular coordinates to latitudinal coordinates (radians).

        [radius, lon, lat] = spy.reclat(pnear_ray[i])

        lon *= 180./sc.pi

        lat *= 180./sc.pi

        d_lon [i] = lon

        d_lat[i] = lat

        d_radius[i] = radius

	# Given an ephemeris epoch, compute the local solar time for an object on the surface of a body at a specified longitude.

        [hr, minute, secend, timee, ampm] = spy.et2lst( ET_MEX_Mars[i], 499, lon, 'PLANETOCENTRIC')

        local_solar_time.append(timee)

# Find the illumination angles at a specified surface point of a target body.

        [phase, solar, emission] = spy.illum('Mars', ET_MEX_Mars[i], 'None', FOLDER_TYPE.name, pnear_ray[i])

        d_phase[i] = phase

        sza = solar * 180./sc.pi

        SZA[i] = sza

    ang_MEX_LoS = np.zeros((N_data,))

    for i in range(N_data):

        vsep = spy.vsep(vel_MEX[i],pos_MEX_Earth[i])

        ang_MEX_LoS[i] = vsep


    dist_Mars_Sun = np.zeros((N_data,))

    for i in range(N_data):

        [ position, lt ] = spy.spkpos( 'Mars', ET[i], 'IAU_MARS', 'none', 'Sun' )

        dist_Mars_Sun[i] = np.linalg.norm(position)*6.68459e-9 # why this factor, (km)-(AU)? Seems so...


    #print('')
    #print('')
    #print('\tR6: Done')
    #print('')
    #print('')

    return d_lat, d_lon, dist_Mars_Sun, spclon_SC, spclat_SC, re, f



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Sep  1 12:14:39 2020

@author: ananya
"""

from pudb import set_trace as bp  # noqa: F401

from radiocc.old import (
    R9_BendAng_ImpParam_up,
    R9_BendAng_ImpParam_dn,
    R10_Avg_BendAng_ImpParam,
    R11_Refractivity_and_Bending_Radius_v2,
    R12_Electron_Density,
    R13_Neutral_Number_Density,
    R14_Plot_check,
)


def run(
    Doppler_debias,
    N_data,
    r_MEX_up,
    z_MEX_up,
    z_GS_up,
    vr_MEX_up,
    vz_MEX_up,
    vr_GS_up,
    vz_GS_up,
    gamma_up,
    beta_e_up,
    delta_s_up,
    r_MEX_dn,
    z_MEX_dn,
    z_GS_dn,
    vr_MEX_dn,
    vz_MEX_dn,
    vr_GS_dn,
    vz_GS_dn,
    gamma_dn,
    beta_e_dn,
    delta_s_dn,
    Bande,
    fsup,
    c,
    Diff_doppler,
    item,
    i_integral,
    i_Surface,
    R_Planet,
    e,
    eps0,
    me,
    distance,
    Doppler,
    PLOT_DIR,
    Threshold_neut_upp,
    CC,
    kB,
    EPHE_DIR,
    NAIF_SC,
    NAIF_P,
    NAIF_GS,
    ET,
    Ref_frame,
    DATA_ID_i_profile,
    DATA_DIR,
    Threshold_Surface
):
    kx = 880.0 / 749.0
    ks = 240.0 / 749.0
    Doppler_debias_dn_iono_x = Doppler_debias / (1.0 + (kx * kx))
    Doppler_debias_up_iono_x = Doppler_debias / (kx + (1.0 / kx))

    Doppler_debias_dn_iono_s = Doppler_debias / (1.0 + (ks ** 2))
    Doppler_debias_up_iono_s = Doppler_debias_up_iono_x

    Doppler_debias_dn_atmo_x = Doppler_debias / 2.0
    Doppler_debias_up_atmo_x = Doppler_debias / (2.0 * kx)

    Doppler_debias_dn_atmo_s = Doppler_debias / 2.0
    Doppler_debias_up_atmo_s = Doppler_debias_up_atmo_x

    # Diff_Doppler_debias_dn_iono_s = Diff_doppler / (1.0 - (9.0 / 121.0))

    if Bande == "X":

        if item == "IONO":

            # R8: bending angle & impact parameter up
            (
                imp_param_up,
                bend_ang_up,
                delta_r_up,
                beta_r_up,
            ) = R9_BendAng_ImpParam_up.Bend_up(
                N_data,
                r_MEX_up,
                z_MEX_up,
                z_GS_up,
                vr_MEX_up,
                vz_MEX_up,
                vr_GS_up,
                vz_GS_up,
                gamma_up,
                beta_e_up,
                delta_s_up,
                Doppler_debias_up_iono_x,
                fsup,
                c,
            )

            # R8: bending angle & impact parameter down
            (
                imp_param_dn,
                bend_ang_dn,
                delta_r_dn,
                beta_r_dn,
            ) = R9_BendAng_ImpParam_dn.Bend_dn(
                N_data,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Doppler_debias_dn_iono_x,
                Bande,
                fsup,
                c,
            )

            # R9: Average bending angle & impact parameter
            imp_param, bend_ang = R10_Avg_BendAng_ImpParam.Avg(
                imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn, Bande
            )

        if item == "ATMO":
            # R8: bending angle & impact parameter up
            (
                imp_param_up,
                bend_ang_up,
                delta_r_up,
                beta_r_up,
            ) = R9_BendAng_ImpParam_up.Bend_up(
                N_data,
                r_MEX_up,
                z_MEX_up,
                z_GS_up,
                vr_MEX_up,
                vz_MEX_up,
                vr_GS_up,
                vz_GS_up,
                gamma_up,
                beta_e_up,
                delta_s_up,
                Doppler_debias_up_atmo_x,
                fsup,
                c,
            )

            # R8: bending angle & impact parameter down
            (
                imp_param_dn,
                bend_ang_dn,
                delta_r_dn,
                beta_r_dn,
            ) = R9_BendAng_ImpParam_dn.Bend_dn(
                N_data,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Doppler_debias_dn_atmo_x,
                Bande,
                fsup,
                c,
            )

            # R9: Average bending angle & impact parameter
            imp_param, bend_ang = R10_Avg_BendAng_ImpParam.Avg(
                imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn, Bande
            )

    if Bande == "S":
        if item == "IONO":
            # R8: bending angle & impact parameter up
            (
                imp_param_up,
                bend_ang_up,
                delta_r_up,
                beta_r_up,
            ) = R9_BendAng_ImpParam_up.Bend_up(
                N_data,
                r_MEX_up,
                z_MEX_up,
                z_GS_up,
                vr_MEX_up,
                vz_MEX_up,
                vr_GS_up,
                vz_GS_up,
                gamma_up,
                beta_e_up,
                delta_s_up,
                Doppler_debias_up_iono_s,
                fsup,
                c,
            )

            # R8: bending angle & impact parameter down
            (
                imp_param_dn,
                bend_ang_dn,
                delta_r_dn,
                beta_r_dn,
            ) = R9_BendAng_ImpParam_dn.Bend_dn(
                N_data,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Doppler_debias_dn_iono_s,
                Bande,
                fsup,
                c,
            )

            # R9: Average bending angle & impact parameter
            imp_param, bend_ang = R10_Avg_BendAng_ImpParam.Avg(
                imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn, Bande
            )

        if item == "ATMO":
            # R8: bending angle & impact parameter up
            (
                imp_param_up,
                bend_ang_up,
                delta_r_up,
                beta_r_up,
            ) = R9_BendAng_ImpParam_up.Bend_up(
                N_data,
                r_MEX_up,
                z_MEX_up,
                z_GS_up,
                vr_MEX_up,
                vz_MEX_up,
                vr_GS_up,
                vz_GS_up,
                gamma_up,
                beta_e_up,
                delta_s_up,
                Doppler_debias_up_atmo_s,
                fsup,
                c,
            )

            # R8: bending angle & impact parameter down
            (
                imp_param_dn,
                bend_ang_dn,
                delta_r_dn,
                beta_r_dn,
            ) = R9_BendAng_ImpParam_dn.Bend_dn(
                N_data,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Doppler_debias_dn_atmo_s,
                Bande,
                fsup,
                c,
            )

            # R9: Average bending angle & impact parameter
            imp_param, bend_ang = R10_Avg_BendAng_ImpParam.Avg(
                imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn, Bande
            )

    if Bande == "Diff":

        if item == "IONO":

            # R8: bending angle & impact parameter down
            (
                imp_param,
                bend_ang,
                delta_r_diff,
                beta_r_diff,
            ) = R9_BendAng_ImpParam_dn.Bend_dn(
                N_data,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Diff_Doppler_debias_dn_iono_s,
                Bande,
                fsup,
                c,
            )

        if item == "ATMO":

            # R8: bending angle & impact parameter down
            imp_param, bend_ang, delta_r_dn, beta_r_dn = R9_BendAng_ImpParam_dn.Bend_dn(
                N_data,
                r_MEX_dn,
                z_MEX_dn,
                z_GS_dn,
                vr_MEX_dn,
                vz_MEX_dn,
                vr_GS_dn,
                vz_GS_dn,
                gamma_dn,
                beta_e_dn,
                delta_s_dn,
                Doppler_debias_dn_atmo_s,
                Bande,
                fsup,
                c,
            )  # NEED TO CHECK????

    # R11
    (
        ref_index,
        refractivity,
        bend_radius,
        Sum_tot,
    ) = R11_Refractivity_and_Bending_Radius_v2.Abel_Analytical(
        N_data, i_integral, i_Surface, imp_param, bend_ang, R_Planet
    )

    if item == "IONO":

        Ne = R12_Electron_Density.Elec(
            N_data, refractivity, e, eps0, me, Bande, fsup, c
        )
        TEC = R12_Electron_Density.TEC_Calc(
            N_data, Ne, i_integral, i_Surface, bend_radius, R_Planet
        )

        R14_Plot_check.PLOT1(
            distance, Doppler, Doppler_debias, refractivity, Ne, PLOT_DIR, item
        )
    if item == "ATMO":

        # R13: neutral density (Level 04)
        Nn, i_neut_upp = R13_Neutral_Number_Density.Neut(
            N_data, bend_radius, refractivity, Threshold_neut_upp, i_Surface, CC, kB
        )
        R14_Plot_check.PLOT2(
            distance, Doppler, Doppler_debias, refractivity, Nn, PLOT_DIR, item
        )

    if item == "IONO":
        return refractivity, Ne, TEC
    if item == "ATMO":
        return refractivity, Nn



import numpy as np
import scipy as sc
import scipy.integrate as sci
import matplotlib.pyplot as plt

def Abel_num(N_data, Corr_ind, i_Surface, imp_param, bend_ang, R_Mars):

    def integrand(x, alpha, aj):

        return alpha / np.sqrt(x**2 - aj**2)

    ref_index    = np.full(N_data, np.nan) #np.array([np.nan for i in range(N_data)], float)
    bend_radius  = np.full(N_data, np.nan)#np.array([np.nan for i in range(N_data)], float)
    refractivity = np.full(N_data, np.nan)#np.array([np.nan for i in range(N_data)], float)
    Sum_tot      = np.full(N_data, np.nan)#np.array([np.nan for i in range(N_data)], float)

    for i in range(Corr_ind, i_Surface):

        aj = imp_param[i]
        alpha_0 = bend_ang[0]
        aj_0 = imp_param[0]
        I0 = (1./np.pi) * sci.quad(integrand, aj_0, np.inf, args=(alpha_0, aj))[0]

        Sum = I0
        for j in range(Corr_ind,i+1):
            alpha = (bend_ang[j] + bend_ang[j-1])/2.0
            Sum += -(1./np.pi) * sci.quad(integrand, imp_param[j], imp_param[j-1], args=(alpha, aj))[0]

        Sum_tot[i] = Sum
        ref_index[i] = np.exp(Sum)
        bend_radius[i] = aj/np.exp(Sum)
        refractivity[i] = (np.exp(Sum) - 1.0)*1E6


    #print(I0, Sum)
    print('\tR11: Done')
    #print('')
    #print('')

    return ref_index, refractivity,bend_radius, Sum_tot



def Abel_Analytical(N_data, Corr_ind, i_Surface, imp_param, bend_ang, R_Mars):

    Sum_tot      = np.full(N_data, np.nan)
    refractivity = np.array([np.nan for i in range(N_data)], float)
    bend_radius  = np.array([np.nan for i in range(N_data)], float)

    A = np.array([np.nan for i in range(N_data)], float)
    B = np.array([np.nan for i in range(N_data)], float)

    for i in range(0,N_data):
        alpha_loc  = bend_ang[i:i+2]
        a_loc      = imp_param[i:i+2]
        A[i], B[i] = np.polyfit(a_loc, alpha_loc, 1)

    for j in range(Corr_ind, i_Surface):

        Sum_1 = 0
        for k in range(Corr_ind, j):
            Sum_1 += A[k] * (np.sqrt(imp_param[k+1]**2 - imp_param[j]**2) - \
                      np.sqrt(imp_param[k]**2 - imp_param[j]**2))\
                      + B[k] * np.log(np.abs((imp_param[k+1] +\
                      np.sqrt(imp_param[k+1]**2 - imp_param[j]**2))/(imp_param[k] +\
                             np.sqrt(imp_param[k]**2 - imp_param[j]**2))))

        refractivity[j] = (np.exp((1./sc.pi) * Sum_1) - 1.0)*1.0e6
        bend_radius[j]  = imp_param[j] / np.exp(Sum_1)
        ref_index=bend_radius

    #print(Corr_ind)
    #print('')
    print('\t REFRACTIVITY AND BENDING RADIUS DONE')
    #print('')
    #print('')
    #plt.plot(refractivity)
    return ref_index,refractivity, bend_radius, Sum_tot


#
#



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Aug 30 19:52:33 2020

@author: ananya
"""

from pudb import set_trace as bp  # noqa

import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as sc

def PLOT_Dop1(distance,Doppler,Doppler_debias, Doppler_biasfit, ET, PLOT_DIR, N_data):
    Doppler=np.array(Doppler)
    Doppler_debias=np.array(Doppler_debias)
    cond_dop = Doppler<1
    cond_deb = Doppler_debias<1

    fig6 = plt.figure(6)
    plt.ion()
    plt.show()
    plt.plot(Doppler,distance/1000,':', color='black',label = 'Raw L2 Data')#(1+880./749.)
    plt.plot(Doppler_debias,distance/1000,':', color='blue', label = 'Debias Raw L2 Data')#(1+880./749.)

    # FIXME:
    # + MEX enables lines below but not MAVEN, do we keep that?
    # + same for comment below 'xlim'
    #
    # Doppler = Doppler[cond_dop]
    # Doppler_debias = Doppler_debias[cond_deb]

    min_x1 = np.nanmedian(Doppler_debias)- sc.median_absolute_deviation(Doppler_debias,nan_policy='omit')*5
    min_x2 = np.nanmedian(Doppler)- sc.median_absolute_deviation(Doppler,nan_policy='omit')*5
    max_x1 = np.nanmedian(Doppler_debias)+ sc.median_absolute_deviation(Doppler_debias,nan_policy='omit')*5
    max_x2 = np.nanmedian(Doppler)+ sc.median_absolute_deviation(Doppler,nan_policy='omit')*5
    min_x = min([min_x1,min_x2])
    max_x = max([max_x1,max_x2])
    plt.xlabel('Doppler Frequency Residuals (Hz)')
    plt.ylabel('Altitude (km)')

    plt.ylim(3400, 4000)
    plt.xlim(min_x, max_x)
    # plt.xlim(-0.2,0.2)
    plt.grid(True)
    plt.legend()
    fig6.set_size_inches(6, 10) #
    plt.savefig(PLOT_DIR+'/'+'Doppler_before_corrtn.png',dpi=600)
    plt.pause(1.0)
    input("Press [enter] to continue.")
    plt.close()

    # FIXME:
    # keep it?

    delET = []#np.full(N_data, np.nan)
    for i in range(N_data):
        delET.append(ET[i] - ET[0])

    fig7 = plt.figure(7)
    plt.ion()
    plt.show()
    plt.plot(delET, Doppler,':', color='black',label = 'Raw L2 Data')#(1+880./749.)
    plt.plot(delET, np.array(Doppler_debias)*100, ':', color='blue', label = 'Debias Raw L2 Data')#(1+880./749.)  

    plt.ylabel('Doppler Frequency Residuals (Hz)')
    plt.xlabel('Time (sec)')

    #plt.xlim(0, 6000)
    plt.ylim(-10,10)
    # plt.xlim(-0.2,0.2)
    plt.grid(True)
    plt.legend()
    fig7.set_size_inches(6, 10) #
    plt.savefig(PLOT_DIR+'/'+'Doppler_ET_before_Corr.png',dpi=600)
    plt.pause(1.0)
    input("Press [enter] to continue.")
    plt.close() 

    return



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Aug 30 20:17:48 2020

@author: ananya
"""

import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as sc

def PLOT1( distance,Doppler,Doppler_debias,refractivity, Ne,PLOT_DIR,item):
    Doppler=np.array(Doppler)
    Doppler_debias=np.array(Doppler_debias)
    cond_dop = Doppler<1 
    cond_deb = Doppler_debias<1

    fig6 = plt.figure(6) 
    plt.ion()
    plt.show()
    plt.plot(Doppler,distance/1000,':', color='black',label = 'Raw L2 Data')#(1+880./749.)
    plt.plot(Doppler_debias,distance/1000,':', color='blue', label = 'Debias Raw L2 Data')#(1+880./749.)
    
    Doppler = Doppler[cond_dop]
    Doppler_debias = Doppler_debias[cond_deb]
        
    min_x1 = np.nanmedian(Doppler_debias)- sc.median_absolute_deviation(Doppler_debias,nan_policy='omit')*5
    min_x2 = np.nanmedian(Doppler)- sc.median_absolute_deviation(Doppler,nan_policy='omit')*5
    max_x1 = np.nanmedian(Doppler_debias)+ sc.median_absolute_deviation(Doppler_debias,nan_policy='omit')*5
    max_x2 = np.nanmedian(Doppler)+ sc.median_absolute_deviation(Doppler,nan_policy='omit')*5    
    min_x = min([min_x1,min_x2])
    max_x = max([max_x1,max_x2])
    plt.xlabel('Doppler Frequency Residuals (Hz)')
    plt.ylabel('Altitude (km)')
    
    plt.ylim(3400, 4000)
    plt.xlim(min_x, max_x)
    plt.grid(True)
    plt.legend()
    fig6.set_size_inches(6, 10) #
    #plt.savefig(PLOT_DIR+'/'+'Doppler-before-corr.png',dpi=600)
    plt.pause(1.0)
    input("Press [enter] to continue.")
    #plt.close()
    
    
    
    fig10 = plt.figure(10)    
    plt.ion()
    plt.show()            
    plt.plot(np.array(refractivity),np.array(distance/1000),':', color='blue', label ='after correction')
                     
    plt.xlabel('Refractivity')
    plt.ylabel('Altitude (km)')
        
    plt.ylim(3400,4000)
    plt.xlim(-0.15,0.15)# S band
    plt.grid(True)
    fig10.set_size_inches(6, 10) #Inch
    
    plt.savefig(PLOT_DIR+'/'+'refractivity-before-corr.png',dpi=600)
    plt.pause(1.0)    
    input("Press [enter] to continue.")
    #plt.close() 
    
    fig3 = plt.figure(3) 
    plt.ion()
    plt.show()
    
    plt.plot(np.array(Ne)*1E-6,np.array(distance/1000), ':', color='blue', label ='after correction')
    plt.xlabel('Electron density (el/cm$^{3}$)')

    plt.ylabel('Altitude (km)')
    
    plt.grid(True)
    fig3.set_size_inches(6, 10)    
    plt.xlim(-5.0e4,15.0e4)
    plt.ylim(3400, 4000)
    
    plt.savefig(PLOT_DIR+'/'+'Ne_before-Corr.png',dpi=600) 
    
    
    plt.pause(1.0)
    input("Press [enter] to continue.")
    #plt.close()
    print('\t PLOT CHECK DONE')
    
    return 

def PLOT2( distance,Doppler,Doppler_debias,refractivity, Nn,PLOT_DIR,item):
    print("I WAS HERE")
    """
    Doppler=np.array(Doppler)
    Doppler_debias=np.array(Doppler_debias)
    cond_dop = Doppler<1 
    cond_deb = Doppler_debias<1

    fig6 = plt.figure(6) 
    plt.ion()
    plt.show()
    plt.plot(Doppler,distance/1000,':', color='black',label = 'Raw L2 Data')#(1+880./749.)
    plt.plot(Doppler_debias,distance/1000,':', color='blue', label = 'Debias Raw L2 Data')#(1+880./749.)
    
    Doppler = Doppler[cond_dop]
    Doppler_debias = Doppler_debias[cond_deb]
        
    min_x1 = np.nanmedian(Doppler_debias)- sc.median_absolute_deviation(Doppler_debias,nan_policy='omit')*5
    min_x2 = np.nanmedian(Doppler)- sc.median_absolute_deviation(Doppler,nan_policy='omit')*5
    max_x1 = np.nanmedian(Doppler_debias)+ sc.median_absolute_deviation(Doppler_debias,nan_policy='omit')*5
    max_x2 = np.nanmedian(Doppler)+ sc.median_absolute_deviation(Doppler,nan_policy='omit')*5    
    min_x = min([min_x1,min_x2])
    max_x = max([max_x1,max_x2])
    plt.xlabel('Doppler Frequency Residuals (Hz)')
    plt.ylabel('Altitude (km)')
    
    plt.ylim(3400, 4000)
    plt.xlim(min_x, max_x)
    plt.grid(True)
    plt.legend()
    fig6.set_size_inches(6, 10) #
    plt.savefig(PLOT_DIR+'/'+'Doppler-before-corr.png',dpi=600)
    plt.pause(1.0)
    input("Press [enter] to continue.")
    #plt.close()
    
    
    
    fig10 = plt.figure(10)    
    plt.ion()
    plt.show()            
    plt.plot(np.array(refractivity),np.array(distance/1000),':', color='blue', label ='after correction')
                     
    plt.xlabel('Refractivity')
    plt.ylabel('Altitude (km)')
        
    plt.ylim(3400,4000)
    plt.xlim(-0.05,0.05)# S band
    plt.grid(True)
    fig10.set_size_inches(6, 10) #Inch
    
    plt.savefig(PLOT_DIR+'/'+'refractivity-before-corr.png',dpi=600)
    plt.pause(1.0)    
    input("Press [enter] to continue.")
    #plt.close()
    """
    fig3 = plt.figure(3) 
    plt.ion()
    plt.show()
    
    plt.plot(np.array(Nn),np.array(distance/1000), ':', color='blue', label ='after correction')
    plt.xlabel('Neutral density (el/cm$^{3}$)')

    plt.ylabel('Altitude (km)')
    
    plt.grid(True)
    fig3.set_size_inches(6, 10)    
    plt.xlim(xmin=-1E22,xmax=3E23)
    #plt.ylim(3400, 4000)
    
    plt.savefig(PLOT_DIR+'/'+'Nn_before-Corr.png',dpi=600) 
    
    
    plt.pause(1.0)
    input("Press [enter] to continue.")
    
    #plt.close()
    print('\t PLOT CHECK DONE')
    
    return 



import numpy as np
import os
import matplotlib.pyplot as plt
import csv
import pylab
import matplotlib.ticker
import sys
import pandas as pd
import sys
import scipy.stats as sc
import matplotlib
import glob

from pudb import set_trace as bp  # noqa


def PLOT(
    DATA_ID,
    i_Profile,
    DATA_PRO,
    Patzold_DIR,
    CODE_DIR,
    Bande,
    DATA_DIR,
    Threshold_Cor,
    Threshold_Surf,
    DATA_FINAL_DIR_ATMO,
    DATA_FINAL_DIR_IONO,
    PLOT_DIR,
    Threshold_int,
):

    ############################## NEW PART #########################
    ############################## PATZOLD FINDER ###################

    DATA = DATA_ID[i_Profile]

    os.listdir(DATA_PRO)
    path = np.array(pd.Series(os.listdir(DATA_PRO)).str.split(",").tolist())
    count = np.where(path == DATA)

    DATA_path = DATA_PRO + "/" + path[count][0]

    TXT = DATA_path + "/AAREADME.TXT"

    info = open(TXT, "r")
    info.readline()
    for line in info:
        if "Data from Experiments" in line:
            the_line = line
    cond = -999
    test = np.array(pd.Series(the_line).str.split("/").tolist())

    day_number = (test[0][1]).astype(np.int).astype(np.string_).decode("UTF-8")
    year = np.array(pd.Series(test[0][0]).str.split(" ").tolist())[0][-1]

    # Go to the good year folder :

    Patzold_iono = Patzold_DIR + "/OCC_IONO/"
    Patzold_atmo = Patzold_DIR + "/OCC_ATMO/"

    index_file_iono = np.where(np.array(os.listdir(Patzold_iono)) == year)
    index_file_atmo = np.where(np.array(os.listdir(Patzold_atmo)) == year)

    if np.shape(index_file_iono)[0] == 0:
        print("No IONO Patzold data file available for this year")
        cond = 0
    else:
        Patzold_iono = Patzold_iono + os.listdir(Patzold_iono)[index_file_iono[0][0]]

    if np.shape(index_file_atmo)[0] == 0:
        print("No ATMO Patzold data file available for this year")
        cond = 0
    else:
        Patzold_atmo = Patzold_atmo + os.listdir(Patzold_atmo)[index_file_atmo[0][0]]

        # Go to the good folder according to the day number
        step = 0
        for file in glob.glob(Patzold_iono + "/" + "*" + str(day_number) + "*"):
            step = step + 1
            if step > 0:
                path_new = file
        step1 = 0
        for file1 in glob.glob(Patzold_atmo + "/" + "*" + str(day_number) + "*"):
            step1 = step1 + 1
            if step1 > 0:
                path_new_1 = file1

        if step == 0:
            print("No IONO Patzold data file available for this orbit")
            cond = 0
        else:
            Patzold_iono = path_new + "/"

        if step1 == 0:
            print("No ATMO Patzold data file available for this orbit")
            cond = 0
        else:
            Patzold_atmo = path_new_1 + "/"

            # Les fichiers avec l'acronyme IIX :
            files_AIX = []
            files_IIX = []

            for file in glob.glob(Patzold_iono + "*IIX*.TAB"):
                files_IIX.append(file)

            for file in glob.glob(Patzold_atmo + "*AIX*.TAB"):
                files_AIX.append(file)
            if np.shape(files_IIX)[0] == 0:
                print("No IONO Patzold processed data file available for this orbit")
                cond = 0
            else:
                path_iono = files_IIX[-1]

            # Les fichiers avec l'acronyme AIX :

            if np.shape(files_AIX)[0] == 0:
                print("No ATMO Patzold processed data file available for this orbit")
                cond = 0
            else:
                path_atmo = files_AIX[-1]

    ###################################### END ####################################

    ########Patzold data#########################################################################

    if cond == 0:
        file_ion = []
    else:
        file_ion = path_iono

    if cond == 0:
        file_atmo = []
    else:
        file_atm = path_atmo

    file_iono = np.genfromtxt(file_ion, dtype=None)
    file_atmo = np.genfromtxt(file_atm, dtype=None)

    ###Loading Python output#################################################################################

    Python_OUT_IONO = []
    N_data_IONO = []

    # output filename
    if Bande == "X":
        Python_OUT_IONO.append(
            DATA_FINAL_DIR_IONO + "/OUT_DPX_IONO_v12a_" + str(i_Profile + 1) + ".txt"
        )
    if Bande == "S":
        Python_OUT_IONO.append(
            DATA_FINAL_DIR_IONO + "/OUT_DPS_IONO_v12a_" + str(i_Profile + 1) + ".txt"
        )
    if Bande == "Diff":
        Python_OUT_IONO.append(
            DATA_FINAL_DIR_IONO + "/OUT_Diff_IONO_v12a_" + str(i_Profile + 1) + ".txt"
        )

    output_data_IONO = np.genfromtxt(Python_OUT_IONO[-1], dtype=None)
    N_data_IONO = len(output_data_IONO)
    Doppler = []
    Doppler_debias = []
    Doppler_biasfit = []
    distance = []
    refractivity1 = []
    Ne = []
    error = []
    error_refrac1 = []
    error_elec = []

    for i in range(N_data_IONO):
        Doppler.append(output_data_IONO[i][0])
        Doppler_debias.append(output_data_IONO[i][1])
        Doppler_biasfit.append(output_data_IONO[i][2])
        distance.append(output_data_IONO[i][3])
        refractivity1.append(output_data_IONO[i][4])
        Ne.append(output_data_IONO[i][5])
        error.append(output_data_IONO[i][6])
        error_refrac1.append(output_data_IONO[i][7])
        error_elec.append(output_data_IONO[i][8])

    Python_OUT_ATMO = []
    N_data_ATMO = []

    # output filename
    if Bande == "X":
        Python_OUT_ATMO.append(
            DATA_FINAL_DIR_ATMO + "/OUT_DPX_ATMO_v12a_" + str(i_Profile + 1) + ".txt"
        )
    if Bande == "S":
        Python_OUT_ATMO.append(
            DATA_FINAL_DIR_ATMO + "/OUT_DPS_ATMO_v12a_" + str(i_Profile + 1) + ".txt"
        )
    if Bande == "Diff":
        Python_OUT_ATMO.append(
            DATA_FINAL_DIR_ATMO + "/OUT_Diff_ATMO_v12a_" + str(i_Profile + 1) + ".txt"
        )
    # print('*** ' + Python_OUT[-1] + '\n')
    output_data_ATMO = np.genfromtxt(Python_OUT_ATMO[-1], dtype=None)
    N_data_ATMO = len(output_data_ATMO)
    Doppler = []
    Doppler_debias = []
    Doppler_biasfit = []
    distance = []
    refractivity2 = []
    Nn = []
    error = []
    error_refrac2 = []
    error_neut = []

    for i in range(N_data_ATMO):
        Doppler.append(output_data_ATMO[i][0])
        Doppler_debias.append(output_data_ATMO[i][1])
        Doppler_biasfit.append(output_data_ATMO[i][2])
        distance.append(output_data_ATMO[i][3] / 1e3)
        refractivity2.append(output_data_ATMO[i][4])
        Nn.append(output_data_ATMO[i][5])
        error.append(output_data_ATMO[i][6])
        error_refrac2.append(output_data_ATMO[i][7])
        error_neut.append(output_data_ATMO[i][8])

    # #########print L2_data#################################################################################
    # os.chdir(DATA_PRO)
    # PATH = os.path.normpath(DATA_PRO + os.sep + os.pardir)

    # if Bande == 'X':
    #     if os.path.getsize(PATH+'/INFORMATION/PATH_X.txt') == 0 :
    #         print('No data available in X')
    #         sys.exit('No data to plot')
    #     else:
    #         file = open(PATH+'/INFORMATION/PATH_X.txt', 'r')
    #         PATH_X = file.read()
    #         L2_data = np.genfromtxt(PATH_X,dtype=None)
    # else :
    #     if os.path.getsize(PATH+'/INFORMATION/PATH_S.txt') == 0 :
    #         print('No data available in S')
    #         sys.exit('No data to plot')
    #     else:
    #         file = open(PATH+'/INFORMATION/PATH_S.txt', 'r')
    #         PATH_S = file.read()
    #         L2_data = np.genfromtxt(PATH_S,dtype=None)

    # Patzold IONO labels
    SAMPLE_NUMBER_IONO = []
    UTC_TIME_IONO = []
    EPHEMERIS_SECONDS_IONO = []
    RADIUS_IONO = []
    REFRACTIVITY_IONO = []
    ELECTRON_NUMBER_DENSITY_IONO = []
    SIGMA_ELECTRON_NUMBER_DENSITY_IONO = []

    for i in range(len(file_iono)):
        SAMPLE_NUMBER_IONO.append(file_iono[i][0])
        UTC_TIME_IONO.append(file_iono[i][1])
        EPHEMERIS_SECONDS_IONO.append((file_iono[i][2]))
        RADIUS_IONO.append((file_iono[i][3]))
        REFRACTIVITY_IONO.append(file_iono[i][7])
        ELECTRON_NUMBER_DENSITY_IONO.append((float(file_iono[i][9])))
        SIGMA_ELECTRON_NUMBER_DENSITY_IONO.append(file_iono[i][10])

    # Patzold ATMO labels
    UTC_TIME_ATMO = []
    EPHEMERIS_SECONDS_ATMO = []
    RADIUS_ATMO = []
    NUMBER_DENSITY_ATMO = []

    for i in range(len(file_atmo)):
        UTC_TIME_ATMO.append(file_atmo[i][1])
        EPHEMERIS_SECONDS_ATMO.append(file_atmo[i][2])
        RADIUS_ATMO.append(file_atmo[i][3])
        NUMBER_DENSITY_ATMO.append(file_atmo[i][20])

    distance = np.array(distance) - Threshold_Surf / 1e3
    RADIUS_IONO = np.array(RADIUS_IONO) - Threshold_Surf / 1e3
    RADIUS_ATMO = np.array(RADIUS_ATMO) - Threshold_Surf / 1e3

    ########Plot files##########################################################################
    # retval = os.getcwd()
    # print("Current working directory %s" % retval)
    # min_y = Threshold_Surf
    # max_y = Threshold_Cor

    # min_y = Threshold_Surf
    # max_y = Threshold_Cor
    Doppler = np.array(Doppler)
    Doppler_debias = np.array(Doppler_debias)
    cond_dop = Doppler < 1
    cond_deb = Doppler_debias < 1

    fig6 = plt.figure(6)
    plt.errorbar(
        Doppler,
        distance,
        xerr=np.array(error),
        fmt="o",
        color="black",
        ecolor="lightgray",
        elinewidth=1,
        capsize=1,
        ms=1,
        label="Raw L2 Data",
    )  # (1+880./749.)
    plt.errorbar(
        Doppler_debias,
        distance,
        xerr=np.array(error),
        fmt="o",
        color="red",
        ecolor="lightgray",
        elinewidth=1,
        capsize=1,
        ms=1,
        label="Debias Raw L2 Data",
    )  # (1+880./749.)

    Doppler = Doppler[cond_dop]
    Doppler_debias = Doppler_debias[cond_deb]

    min_x1 = (
        np.nanmedian(Doppler_debias)
        - sc.median_absolute_deviation(Doppler_debias, nan_policy="omit") * 5
    )
    min_x2 = (
        np.nanmedian(Doppler)
        - sc.median_absolute_deviation(Doppler, nan_policy="omit") * 5
    )
    max_x1 = (
        np.nanmedian(Doppler_debias)
        + sc.median_absolute_deviation(Doppler_debias, nan_policy="omit") * 5
    )
    max_x2 = (
        np.nanmedian(Doppler)
        + sc.median_absolute_deviation(Doppler, nan_policy="omit") * 5
    )
    min_x = min([min_x1, min_x2])
    max_x = max([max_x1, max_x2])

    # plt.plot(Wang_dig_1, Wang_dig_2, label = 'Wang et al. ')
    plt.xlabel("Doppler Frequency Residuals (Hz)")
    plt.ylabel("Altitude (km)")
    plt.xlim(min_x, max_x)
    plt.ylim(ymin=0)
    plt.legend()
    plt.grid(True)
    fig6.set_size_inches(8, 6)  # Inch
    plt.close(fig6)
    ############################ REFRACTIVITY ATMO

    fig20 = plt.figure(20)
    cond_ref2 = np.where(np.array(refractivity2) > -1)
    if Bande == "X":
        plt.errorbar(
            np.array(refractivity2)[cond_ref2],
            np.array(distance)[cond_ref2],
            xerr=np.array(error_refrac2)[cond_ref2],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data",
        )

    if Bande == "S":

        plt.errorbar(
            np.array(refractivity2)[cond_ref2],
            np.array(distance)[cond_ref2],
            xerr=np.array(error_refrac2)[cond_ref2],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data",
        )

    if Bande == "Diff":

        plt.errorbar(
            np.array(refractivity2)[cond_ref2],
            np.array(distance)[cond_ref2],
            xerr=np.array(error_refrac2)[cond_ref2],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data",
        )

    plt.xlabel("Refractivity")
    plt.ylabel("Altitude (km)")
    pylab.legend(loc="upper right")

    min_x = np.nanmin(np.array(refractivity2)[cond_ref2])
    max_x = (
        np.nanmedian(np.array(refractivity2)[cond_ref2])
        + sc.median_absolute_deviation(
            np.array(refractivity2)[cond_ref2], nan_policy="omit"
        )
        * 15
    )

    plt.xlim(min_x, max_x)
    plt.grid(True)
    plt.legend()
    fig20.set_size_inches(8, 6)  # Inch

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "refractivity_ATMO_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "refractivity_ATMO_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "refractivity_ATMO_Diff.png", dpi=600)

    plt.close(fig20)

    ########################## REFRACTIVITY IONO

    fig10 = plt.figure(10)
    cond_ref1 = np.where(np.array(refractivity1) > -1)

    if Bande == "X":
        plt.errorbar(
            np.array(refractivity1)[cond_ref1],
            np.array(distance)[cond_ref1],
            xerr=np.array(error_refrac1)[cond_ref1],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data",
        )

    if Bande == "S":
        plt.errorbar(
            np.array(refractivity1)[cond_ref1],
            np.array(distance)[cond_ref1],
            xerr=np.array(error_refrac1)[cond_ref1],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data",
        )

    if Bande == "Diff":
        plt.errorbar(
            np.array(refractivity1)[cond_ref1],
            np.array(distance)[cond_ref1],
            xerr=np.array(error_refrac1)[cond_ref1],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler  Data",
        )

    plt.xlabel("Refractivity")
    plt.ylabel("Altitude (km)")
    pylab.legend(loc="upper right")

    min_x = np.nanmin(np.array(refractivity1)[cond_ref1])
    max_x = (
        np.nanmedian(np.array(refractivity1)[cond_ref1])
        + sc.median_absolute_deviation(
            np.array(refractivity1)[cond_ref1], nan_policy="omit"
        )
        * 15
    )

    plt.xlim(min_x, max_x)
    #    plt.ylim(min_y,max_y)
    #    mf = matplotlib.ticker.ScalarFormatter(useOffset=False, useMathText=True)
    #    mf.set_powerlimits((-3,3))
    #    plt.gca().xaxis.set_major_formatter(mf)
    plt.grid(True)
    fig10.set_size_inches(8, 6)  # Inch

    if Bande == "X":
        if cond != 0:
            plt.plot(
                np.array(
                    REFRACTIVITY_IONO[int(len(np.array(REFRACTIVITY_IONO)) / 2) :]
                ),
                RADIUS_IONO[int(len(np.array(REFRACTIVITY_IONO)) / 2) :],
                ":g",
                label="Patzold et al.",
            )

    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "refractivity_IONO_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "refractivity_IONO_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "refractivity_IONO_Diff.png", dpi=600)

    plt.close(fig10)

    ########################## Electron Density
    fig8 = plt.figure(8)

    if Bande == "X":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data" + year + "" + day_number,
        )

    if Bande == "S":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data" + year + "" + day_number,
        )

    if Bande == "Diff":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler" + year + "" + day_number,
        )

    plt.xlabel("Electron density (el/m$^{3}$)")
    plt.ylabel("Altitude (km)")
    pylab.legend(loc="upper right")

    min_x = np.nanmedian(Ne) - sc.median_absolute_deviation(Ne, nan_policy="omit") * 3
    max_x = (
        np.nanmax(np.array(Ne)[np.array(Ne) < 1e13])
        + sc.median_absolute_deviation(
            np.array(Ne)[np.array(Ne) < 1e13], nan_policy="omit"
        )
        * 15
    )

    plt.xlim(min_x, max_x)
    #    plt.ylim(min_y,max_y)
    #    mf = matplotlib.ticker.ScalarFormatter(useOffset=False, useMathText=True)
    #    mf.set_powerlimits((-3,3))
    #    plt.gca().xaxis.set_major_formatter(mf)
    plt.grid(True)
    fig8.set_size_inches(8, 6)  # Inch

    if Bande == "X":
        if cond != 0:
            plt.plot(
                np.array(ELECTRON_NUMBER_DENSITY_IONO) * 1e6,
                RADIUS_IONO,
                ":g",
                label="Patzold et al.",
            )
            # plt.plot(Wang_X_Ne, Wang_X_alt, 'k', label = 'Wang et al.')

    plt.legend

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Ne_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "Ne_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "Ne_Diff.png", dpi=600)
    plt.close(fig8)

    fig11 = plt.figure(11)
    xmini = 10 ** (10)
    index = np.where(np.array(Ne) > xmini)
    if Bande == "X":
        plt.errorbar(
            np.array(Ne[index[0][0] : index[0][-1]]),
            distance[index[0][0] : index[0][-1]],
            xerr=error_elec[index[0][0] : index[0][-1]],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data" + year + "" + day_number,
        )

    if Bande == "S":
        plt.errorbar(
            np.array(Ne[index[0][0] : index[0][-1]]),
            distance[index[0][0] : index[0][-1]],
            xerr=error_elec[index[0][0] : index[0][-1]],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data" + year + "" + day_number,
        )

    if Bande == "Diff":
        plt.errorbar(
            np.array(Ne[index[0][0] : index[0][-1]]),
            distance[index[0][0] : index[0][-1]],
            xerr=error_elec[index[0][0] : index[0][-1]],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data" + year + "" + day_number,
        )
    plt.xlabel("Electron density (el/m$^{3}$)")
    plt.ylabel("Altitude (km)")
    pylab.legend(loc="upper right")

    # plt.xlim(min_x, 1.5E11)
    #    plt.ylim(min_y,max_y)
    #    mf = matplotlib.ticker.ScalarFormatter(useOffset=False, useMathText=True)
    #    mf.set_powerlimits((-3,3))
    #    plt.gca().xaxis.set_major_formatter(mf)
    plt.grid(True)
    plt.legend()
    fig11.set_size_inches(8, 6)  # Inch
    plt.xscale("log")
    plt.xlim(xmin=xmini)
    if cond != 0:
        plt.plot(
            np.array(ELECTRON_NUMBER_DENSITY_IONO) * 1e6,
            RADIUS_IONO,
            ":g",
            label="Patzold et al.",
        )
    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Ne_X_log.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "Ne_S_log.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "Ne_Diff_log.png", dpi=600)
    plt.close(fig11)

    ########################## Neutral Density

    fig9 = plt.figure(9)
    if Bande == "X":
        plt.errorbar(
            np.array(Nn),
            distance,
            xerr=error_neut,
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data" + year + "" + day_number,
        )  # (1+880./749.)

    if Bande == "S":
        plt.errorbar(
            np.array(Nn),
            distance,
            xerr=error_neut,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data" + year + "" + day_number,
        )  # (1+880./749.)

    if Bande == "Diff":
        plt.errorbar(
            np.array(Nn),
            distance,
            xerr=error_neut,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data" + year + "" + day_number,
        )  # (1+880./749.)

    plt.xlabel("Neutral number density (m$^{-3}$)")
    plt.ylabel("Altitude (km)")
    pylab.legend(loc="upper right")

    min_x = np.nanmedian(Nn) - sc.median_absolute_deviation(Nn, nan_policy="omit") * 3
    max_x = np.nanmedian(Nn) + sc.median_absolute_deviation(Nn, nan_policy="omit") * 9

    plt.xlim(xmin=-1e22, xmax=3e23)
    #    plt.ylim(min_y,max_y)
    fig9.set_size_inches(8, 6)  # Inch
    plt.grid(True)
    if cond != 0:
        plt.plot(
            np.array(NUMBER_DENSITY_ATMO),
            np.array(RADIUS_ATMO),
            ":g",
            label="Patzold et al",
        )
    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Neutral_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "Neutral_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "Neutral_Diff.png", dpi=600)
    plt.close(fig9)

    print("\t PLOTS DONE")
    return


def PLOT2(
    DATA_ID,
    i_Profile,
    DATA_PRO,
    CODE_DIR,
    Bande,
    DATA_DIR,
    Threshold_Cor,
    Threshold_Surf,
    DATA_FINAL_DIR,
    PLOT_DIR,
    Threshold_int,
):

    ############################## NEW PART #########################
    ############################## PATZOLD FINDER ###################

    DATA = DATA_ID[i_Profile]

    ###Loading Python output#################################################################################

    Python_OUT_IONO = []
    N_data_IONO = []

    # output filename
    Python_OUT_IONO.append(
        DATA_FINAL_DIR + "/" + Bande + "-IONO-" + str(i_Profile + 1) + ".txt"
    )

    output_data_IONO = np.genfromtxt(Python_OUT_IONO[-1], dtype=None)
    N_data_IONO = len(output_data_IONO)
    Doppler = []
    Doppler_debias = []
    Doppler_biasfit = []
    distance = []
    refractivity1 = []
    Ne = []
    error = []
    error_refrac1 = []
    error_elec = []

    for i in range(N_data_IONO):
        Doppler.append(output_data_IONO[i][0])
        Doppler_debias.append(output_data_IONO[i][1])
        Doppler_biasfit.append(output_data_IONO[i][2])
        distance.append(output_data_IONO[i][3])
        refractivity1.append(output_data_IONO[i][4])
        Ne.append(output_data_IONO[i][5])
        error.append(output_data_IONO[i][6])
        error_refrac1.append(output_data_IONO[i][7])
        error_elec.append(output_data_IONO[i][8])

    Python_OUT_ATMO = []
    N_data_ATMO = []

    # output filename
    Python_OUT_ATMO.append(
        DATA_FINAL_DIR + "/" + Bande + "-ATMO-" + str(i_Profile + 1) + ".txt"
    )

    # print('*** ' + Python_OUT[-1] + '\n')
    output_data_ATMO = np.genfromtxt(Python_OUT_ATMO[-1], dtype=None)
    N_data_ATMO = len(output_data_ATMO)
    Doppler = []
    Doppler_debias = []
    Doppler_biasfit = []
    distance = []
    refractivity2 = []
    Nn = []
    error = []
    error_refrac2 = []
    error_neut = []

    for i in range(N_data_ATMO):
        Doppler.append(output_data_ATMO[i][0])
        Doppler_debias.append(output_data_ATMO[i][1])
        Doppler_biasfit.append(output_data_ATMO[i][2])
        distance.append(output_data_ATMO[i][3])
        refractivity2.append(output_data_ATMO[i][4])
        Nn.append(output_data_ATMO[i][5])
        error.append(output_data_ATMO[i][6])
        error_refrac2.append(output_data_ATMO[i][7])
        error_neut.append(output_data_ATMO[i][8])

    # #########print L2_data#################################################################################
    # os.chdir(DATA_PRO)
    # PATH = os.path.normpath(DATA_PRO + os.sep + os.pardir)

    # if Bande == 'X':
    #     if os.path.getsize(PATH+'/INFORMATION/PATH_X.txt') == 0 :
    #         print('No data available in X')
    #         sys.exit('No data to plot')
    #     else:
    #         file = open(PATH+'/INFORMATION/PATH_X.txt', 'r')
    #         PATH_X = file.read()
    #         L2_data = np.genfromtxt(PATH_X,dtype=None)
    # else :
    #     if os.path.getsize(PATH+'/INFORMATION/PATH_S.txt') == 0 :
    #         print('No data available in S')
    #         sys.exit('No data to plot')
    #     else:
    #         file = open(PATH+'/INFORMATION/PATH_S.txt', 'r')
    #         PATH_S = file.read()
    #         L2_data = np.genfromtxt(PATH_S,dtype=None)

    ########Plot files##########################################################################
    # retval = os.getcwd()
    # print("Current working directory %s" % retval)

    distance = (np.array(distance) - Threshold_Surf) / 1e3
    # min_y = Threshold_Surf
    # max_y = Threshold_Cor

    # min_y = Threshold_Surf
    # max_y = Threshold_Cor
    Doppler = np.array(Doppler)
    Doppler_debias = np.array(Doppler_debias)
    cond_dop = np.logical_and(
        Doppler < 1, distance <= ((Threshold_int - Threshold_Surf) / 1e3)
    )
    cond_deb = np.logical_and(
        Doppler_debias < 1, distance <= ((Threshold_int - Threshold_Surf) / 1e3)
    )

    fig6 = plt.figure(6)
    plt.errorbar(
        Doppler,
        distance,
        xerr=np.array(error),
        fmt="o",
        color="black",
        ecolor="lightgray",
        elinewidth=1,
        capsize=1,
        ms=1,
        label="Raw L2 Data " + DATA,
    )  # (1+880./749.)
    plt.errorbar(
        Doppler_debias,
        distance,
        xerr=np.array(error),
        fmt="o",
        color="red",
        ecolor="lightgray",
        elinewidth=1,
        capsize=1,
        ms=1,
        label="Debias Raw L2 Data " + DATA,
    )  # (1+880./749.)

    Doppler1 = Doppler[cond_dop]
    Doppler_debias1 = Doppler_debias[cond_deb]

    min_x1 = (
        np.nanmedian(Doppler_debias1)
        - sc.median_absolute_deviation(Doppler_debias1, nan_policy="omit") * 5
    )
    min_x2 = (
        np.nanmedian(Doppler1)
        - sc.median_absolute_deviation(Doppler1, nan_policy="omit") * 5
    )
    max_x1 = (
        np.nanmedian(Doppler_debias1)
        + sc.median_absolute_deviation(Doppler_debias1, nan_policy="omit") * 5
    )
    max_x2 = (
        np.nanmedian(Doppler1)
        + sc.median_absolute_deviation(Doppler1, nan_policy="omit") * 5
    )
    min_x = min([min_x1, min_x2])
    max_x = max([max_x1, max_x2])
    max_y = (Threshold_int - Threshold_Surf) / 1e3
    # plt.plot(Wang_dig_1, Wang_dig_2, label = 'Wang et al. ')
    plt.xlabel("Doppler Frequency Residuals (Hz)", fontsize=13)
    plt.ylabel("Altitude (km)", fontsize=13)
    plt.xlim(min_x, max_x)
    plt.ylim(ymin=0, ymax=max_y)
    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)
    #    plt.ylim(min_y,max_y)
    plt.grid(True)
    plt.legend()
    fig6.set_size_inches(8, 6)  # Inch

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Doppler_X1.png", dpi=600)
    else:
        plt.savefig(PLOT_DIR + "/" + "Doppler_S1.png", dpi=600)
    plt.close(fig6)

    """
    fig7 = plt.figure(7)
    plt.errorbar(Doppler,distance,xerr=np.array(error), fmt='o', color='black',
                 ecolor='lightgray', elinewidth=1, capsize=1, ms = 1, label = 'Raw L2 Data '+DATA)#(1+880./749.)

    plt.xlabel('Doppler Frequency Residuals (Hz)',fontsize=13)
    plt.ylabel('Altitude (km)',fontsize=13)
    plt.xlim(min_x, max_x)
    plt.ylim(ymin=0,ymax=max_y)
    plt.legend()
    fig7.set_size_inches(8, 6) #Inch
    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)
    plt.grid(True)
    if Bande == 'X':
        plt.savefig(PLOT_DIR + '/' + 'Doppler_X2.png',dpi=600)
    else :
        plt.savefig(PLOT_DIR + '/' + 'Doppler_S2.png',dpi=600)
    plt.close(fig7)
    """
    ############################ REFRACTIVITY ATMO

    fig20 = plt.figure(20)
    cond_ref2 = np.where(np.array(refractivity2) > -1)

    if Bande == "X":
        plt.errorbar(
            np.array(refractivity2)[cond_ref2],
            np.array(distance)[cond_ref2],
            xerr=np.array(error_refrac2)[cond_ref2],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data " + DATA,
        )

    if Bande == "S":

        plt.errorbar(
            np.array(refractivity2)[cond_ref2],
            distance[cond_ref2],
            xerr=np.array(error_refrac2)[cond_ref2],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data " + DATA,
        )

    if Bande == "Diff":

        plt.errorbar(
            np.array(refractivity2)[cond_ref2],
            distance[cond_ref2],
            xerr=np.array(error_refrac2)[cond_ref2],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data " + DATA,
        )

    plt.xlabel("Refractivity", fontsize=13)
    plt.ylabel("Altitude (km)", fontsize=13)
    pylab.legend(loc="upper right")

    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)

    min_x = np.nanmin(np.array(refractivity2)[cond_ref2])
    max_x = (
        np.nanmedian(np.array(refractivity2)[cond_ref2])
        + sc.median_absolute_deviation(
            np.array(refractivity2)[cond_ref2], nan_policy="omit"
        )
        * 3
    )

    plt.xlim(min_x, max_x)
    plt.grid(True)
    plt.legend()
    fig20.set_size_inches(8, 6)  # Inch

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "refractivity_ATMO_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "refractivity_ATMO_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "refractivity_ATMO_Diff.png", dpi=600)

    plt.close(fig20)

    ########################## REFRACTIVITY IONO

    fig10 = plt.figure(10)
    cond_ref1 = np.where(np.array(refractivity1) > -1)
    if Bande == "X":
        plt.errorbar(
            np.array(refractivity1)[cond_ref1],
            np.array(distance)[cond_ref1],
            xerr=np.array(error_refrac1)[cond_ref1],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data " + DATA,
        )

    if Bande == "S":
        plt.errorbar(
            np.array(refractivity1)[cond_ref1],
            np.array(distance)[cond_ref1],
            xerr=np.array(error_refrac1)[cond_ref1],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data " + DATA,
        )

    if Bande == "Diff":
        plt.errorbar(
            np.array(refractivity1)[cond_ref1],
            np.array(distance)[cond_ref1],
            xerr=np.array(error_refrac1)[cond_ref1],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data " + DATA,
        )

    plt.xlabel("Refractivity", fontsize=13)
    plt.ylabel("Altitude (km)", fontsize=13)
    pylab.legend(loc="upper right")

    min_x = np.nanmin(np.array(refractivity1)[cond_ref1])
    max_x = (
        np.nanmedian(np.array(refractivity1)[cond_ref1])
        + sc.median_absolute_deviation(
            np.array(refractivity1)[cond_ref1], nan_policy="omit"
        )
        * 3
    )

    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)

    # plt.xlim(min_x, max_x)
    #    plt.ylim(min_y,max_y)
    #    mf = matplotlib.ticker.ScalarFormatter(useOffset=False, useMathText=True)
    #    mf.set_powerlimits((-3,3))
    #    plt.gca().xaxis.set_major_formatter(mf)
    plt.grid(True)
    fig10.set_size_inches(8, 6)  # Inch
    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "refractivity_IONO_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "refractivity_IONO_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "refractivity_IONO_Diff.png", dpi=600)

    plt.close(fig10)

    ########################Electron Density
    fig8 = plt.figure(8)

    if Bande == "X":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data " + DATA,
        )

    if Bande == "S":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data " + DATA,
        )

    if Bande == "Diff":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Data " + DATA,
        )

    plt.xlabel("Electron density (el/m$^{3}$)", fontsize=13)

    #    plt.ylim(min_y,max_y)    plt.ylabel('Altitude (km)',fontsize=13)
    pylab.legend(loc="upper right")
    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)

    min_x = np.nanmedian(Ne) - sc.median_absolute_deviation(Ne, nan_policy="omit") * 3
    max_x = (
        np.nanmax(np.array(Ne)[np.array(Ne) < 1e13])
        + sc.median_absolute_deviation(
            np.array(Ne)[np.array(Ne) < 1e13], nan_policy="omit"
        )
        * 3
    )
    # plt.xlim(min_x, max_x)
    #    mf = matplotlib.ticker.ScalarFormatter(useOffset=False, useMathText=True)
    #    mf.set_powerlimits((-3,3))
    #    plt.gca().xaxis.set_major_formatter(mf)
    plt.grid(True)
    fig8.set_size_inches(8, 6)  # Inch
    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Ne_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "Ne_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "Ne_Diff.png", dpi=600)
    plt.close(fig8)

    fig11 = plt.figure(11)
    xmini = 10 ** 8
    xmax = max_x
    index = np.where(np.array(Ne) > xmini)

    if Bande == "X":
        plt.errorbar(
            np.array(Ne[index[0][0] : index[0][-1]]),
            distance[index[0][0] : index[0][-1]],
            xerr=error_elec[index[0][0] : index[0][-1]],
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data " + DATA,
        )

    if Bande == "S":
        plt.errorbar(
            np.array(Ne[index[0][0] : index[0][-1]]),
            distance[index[0][0] : index[0][-1]],
            xerr=error_elec[index[0][0] : index[0][-1]],
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data " + DATA,
        )

    if Bande == "Diff":
        plt.errorbar(
            np.array(Ne),
            distance,
            xerr=error_elec,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data" + DATA,
        )

    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)

    plt.xlabel("Electron density (el/m$^{3}$)", fontsize=13)
    plt.ylabel("Altitude (km)", fontsize=13)
    pylab.legend(loc="upper right")

    # plt.xlim(min_x, 1.5E11)
    #    plt.ylim(min_y,max_y)
    #    mf = matplotlib.ticker.ScalarFormatter(useOffset=False, useMathText=True)
    #    mf.set_powerlimits((-3,3))
    #    plt.gca().xaxis.set_major_formatter(mf)
    plt.grid(True)
    plt.legend()
    fig11.set_size_inches(8, 6)  # Inch
    plt.xscale("log")
    plt.xlim(xmini, xmax)
    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Ne_X_log.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "Ne_S_log.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "Ne_Diff_log.png", dpi=600)
    plt.close(fig11)

    ########################Neutral Density
    fig9 = plt.figure(9)
    if Bande == "X":
        plt.errorbar(
            np.array(Nn),
            distance,
            xerr=error_neut,
            fmt="o",
            color="black",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="X-Band Data " + DATA,
        )  # (1+880./749.)

    if Bande == "S":
        plt.errorbar(
            np.array(Nn),
            distance,
            xerr=error_neut,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="S-Band Data " + DATA,
        )  # (1+880./749.)

    if Bande == "Diff":
        plt.errorbar(
            np.array(Nn),
            distance,
            xerr=error_neut,
            fmt="o",
            color="blue",
            ecolor="lightgray",
            elinewidth=1,
            capsize=1,
            ms=1,
            label="Differential Doppler Data" + DATA,
        )  # (1+880./749.)

    plt.xlabel("Neutral number density (m$^{-3}$)", fontsize=13)
    plt.ylabel("Altitude (km)", fontsize=13)
    pylab.legend(loc="upper right")
    plt.xticks(fontsize=13)
    plt.yticks(fontsize=13)

    min_x = np.nanmedian(Nn) - sc.median_absolute_deviation(Nn, nan_policy="omit") * 3
    max_x = np.nanmedian(Nn) + sc.median_absolute_deviation(Nn, nan_policy="omit") * 9

    plt.grid(True)
    plt.xlim(xmin=-1e22, xmax=3e23)
    #    plt.ylim(min_y,max_y)
    fig9.set_size_inches(8, 6)  # Inch
    plt.legend()

    if Bande == "X":
        plt.savefig(PLOT_DIR + "/" + "Neutral_X.png", dpi=600)
    if Bande == "S":
        plt.savefig(PLOT_DIR + "/" + "Neutral_S.png", dpi=600)
    if Bande == "Diff":
        plt.savefig(PLOT_DIR + "/" + "Neutral_Diff.png", dpi=600)
    plt.close(fig9)

    print("\t PLOTS DONE")
    return



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Nov  5 15:26:40 2019

@author: ananya
"""

import numpy as np
import spiceypy as spy
import os

from pudb import set_trace as bp  # noqa

from radiocc.model import ProcessType


def ephemerides(
    N_data, ET, SC, GS, Mars, DATA_DIR, Ref_frame, FOLDER_TYPE: ProcessType
):
    # os.chdir(DATA_DIR)
    # tB          = np.zeros((N_data,))
    tMEX = np.zeros((N_data,))
    tA = np.zeros((N_data,))
    t0_up = np.zeros((N_data,))
    t0_dn = np.zeros((N_data,))
    t0_upp = np.zeros((N_data,))

    #
    #
    #
    #
    #
    #
    #
    #
    #    GS(to_up) ---> t_Mex ----> G(to_dn)

    ET_MEX_Mars = np.zeros((N_data,))

    pos_GS_dn = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    vel_GS_dn = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    pos_MEX_dn = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    vel_MEX_dn = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    pos_GS_up = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    vel_GS_up = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    pos_MEX_up = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    vel_MEX_up = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    pos_GS_upp = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    vel_GS_upp = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    pos_MEX_upp = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)
    vel_MEX_upp = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], dtype = float)

    pos_MEX = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], float)
    vel_MEX = np.full(
        (N_data, 3), np.nan
    )  # np.array([[np.nan for j in range(3)] for i in range(N_data)], float)

    # This routine computes the transmit (or receive) time  of a signal at a specified target, given the receive  (or transmit) time at a specified observer. The elapsed   time between transmit and receive is also returned.
    #  Direction the signal travels ( "->" or "<-" )
    # etobs      I   Epoch of a signal at some observer
    # obs        I   NAIF ID of some observer
    # dir        I   Direction the signal travels ( "->" or "<-" )
    # targ       I   NAIF ID of the target object
    # ettarg     O   Epoch of the signal at the target
    # elapsd     O   Time between transmit and receipt of the signal

    # void ltime_c ( etobs, obs, ConstSpiceChar   * dir, SpiceInt           targ,
    #               output    * ettarg,
    #               output     * elapsd  )

    # spkezr Return the state (position and velocity) of a target body  relative to an observing body, optionally corrected for light   time (planetary aberration) and stellar aberration.
    # void spkezr_c (targ, et,  ref, abcorr, obs) output  starg[6],*lt        )

    # SUBROUTINE LTIME ( Epoch of a signal at some observer, NAIF-id of some observer, direction the signal travel, AIF-id of the target object,  Time between transmit and receipt of the signal )

    for i in range(N_data):
        # Downlink
        # tB = ET[i]
        [ET_MEX_GS, t_0] = spy.ltime(
            ET[i], int(GS), "<-", int(SC)
        )  #  Epoch of the signal at  MEX tdown
        tMEX[
            i
        ] = ET_MEX_GS  # time at which MEX send signal to mars via downlink = time at which MEX received signal from Mars via uplink

        #  [ET_Mars_GS, t_0] = spy.ltime(tMEX[i], int(Mars), '<-', int(SC)) #  Epoch of the signal at  Mars t down

        [ET_Mars_GS, t_0] = spy.ltime(
            tMEX[i], int(SC), "->", int(Mars)
        )  #  Epoch of the signal at  Mars t down
        t0_dn[i] = ET_Mars_GS  # The signal was transmitted at: t0_dn from Mars

        # [ET_Mars_GS, t_0] = spy.ltime(tMEX[i], int(Mars), '->', int(SC)) #  Epoch of the signal at  MARS tup
        [ET_Mars_GS, t_0] = spy.ltime(
            tMEX[i], int(SC), "<-", int(Mars)
        )  #  Epoch of the signal at  Mars tup
        t0_up[i] = ET_Mars_GS  # time at which Mars send signal to MEX uplink

        # [ states1, lt1 ] = spy.spkezr( 'DSS-65', t0_dn[i] , 'marsiau', "NONE", 'mars' )
        [states1, lt1] = spy.spkezr(
            GS, t0_dn[i], Ref_frame, "NONE", "mars"
        )  #  position of GS at  T_Mars wrt Mars
        pos_GS_dn[i] = states1[:3] * 1e3
        vel_GS_dn[i] = states1[3:] * 1e3

        [states2, lt2] = spy.spkezr(
            FOLDER_TYPE.name, t0_dn[i], Ref_frame, "NONE", "mars"
        )  #  position of MEX at  T_Mars wrt Mars
        pos_MEX_dn[i] = states2[:3] * 1e3
        vel_MEX_dn[i] = states2[3:] * 1e3

        # uplink

        [states3, lt3] = spy.spkezr(
            GS, t0_up[i], Ref_frame, "NONE", "mars"
        )  # position of  GS tup
        pos_GS_up[i] = states3[:3] * 1e3
        vel_GS_up[i] = states3[3:] * 1e3

        [states4, lt2] = spy.spkezr(
            FOLDER_TYPE.name, t0_up[i], Ref_frame, "NONE", "mars"
        )  # position of  MEX tup
        pos_MEX_up[i] = states4[:3] * 1e3
        vel_MEX_up[i] = states4[3:] * 1e3

        [ET_GS_SC, t_0] = spy.ltime(tMEX[i], int(SC), "<-", int(GS))
        tA[i] = ET_GS_SC

        # Used for routine R6, not in R9 !

        [ET_MEX_Marss, one_way_t_span] = spy.ltime(tMEX[i], int(Mars), "<-", int(SC))
        ET_MEX_Mars[i] = ET_MEX_Marss

        [states5, lt] = spy.spkezr(
            FOLDER_TYPE.name, ET_MEX_Mars[i], Ref_frame, "NONE", "mars"
        )
        pos_MEX[i] = states5[:3] * 1e3
        vel_MEX[i] = states5[3:] * 1e3

        [ET_GS_Mars, t_0] = spy.ltime(tA[i], int(Mars), "<-", int(GS))
        t0_upp[i] = ET_GS_Mars

        [states6, lt6] = spy.spkezr(GS, t0_upp[i], Ref_frame, "NONE", "mars")
        pos_GS_upp[i] = states6[:3] * 1e3
        vel_GS_upp[i] = states6[3:] * 1e3

        [states7, lt7] = spy.spkezr(
            FOLDER_TYPE.name, t0_upp[i], Ref_frame, "NONE", "mars"
        )
        pos_MEX_upp[i] = states7[:3] * 1e3
        vel_MEX_upp[i] = states7[3:] * 1e3

    # print('\tR5: Done')
    # print('')
    # print('')
    # print('')
    # print('')

    return (
        t0_up,
        pos_GS_up,
        vel_GS_up,
        pos_MEX_up,
        vel_MEX_up,
        tMEX,
        t0_dn,
        pos_GS_dn,
        vel_GS_dn,
        pos_MEX_dn,
        vel_MEX_dn,
        t0_upp,
        pos_GS_upp,
        vel_GS_upp,
        pos_MEX_upp,
        vel_MEX_upp,
        pos_MEX,
        vel_MEX,
        ET_MEX_Mars,
    )
